package br.com.centralit.api.dao.impl;

import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.SolicitacaoAlcadaDao;
import br.com.centralit.api.model.Alcada;
import br.com.centralit.api.model.CentroResultado;
import br.com.centralit.api.model.NivelAutoridade;
import br.com.centralit.api.model.SolicitacaoAlcada;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.Dominio;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 18/06/2015 - 16:57:12
 *
 * @version 1.0.0
 *
 * @author lucas.ribeiro - (<a href="mailto:lucas.centralit@gmail.com">lucas.centralit@gmail.com</a>)
 *	
 */
@Repository("solicitacaoAlcadaDao")
public class SolicitacaoAlcadaDaoHibernate extends CitGenericDAOImpl implements SolicitacaoAlcadaDao {
	public SolicitacaoAlcadaDaoHibernate() {
		super(SolicitacaoAlcada.class);
	}

	@SuppressWarnings("unchecked")
	@Override
	public Float getValorAcumulado(final Integer ano, final Integer mes, Alcada alcada, CentroResultado centroResultado, Dominio tipoUtilizacao, NivelAutoridade nivelAutoridade) {
		String queryString =  "select solicitacaoAlcada from SolicitacaoAlcada as solicitacaoAlcada"
							+ " where alcada.id = :alcada_id "
							+ "   and centroResultado.id = :centroResultado_id "
							+ "   and tipoUtilizacao.id = :tipoUtilizacao_id "
							+ "   and situacao.nome = 'APROVADA' "
							+ "   and dataInativo is null "
							+ "   and aprovador.nivelAutoridade.id = :nivelAutoridade_id "
							+ "   and ano = :ano ";
		if (mes != null) {
			queryString += " and mes = :mes ";
		}
		Query query = em().createQuery(queryString);
		query.setParameter("alcada_id", alcada.getId());
		query.setParameter("centroResultado_id", centroResultado.getId());
		query.setParameter("tipoUtilizacao_id", tipoUtilizacao.getId());
		query.setParameter("nivelAutoridade_id", nivelAutoridade.getId());
		query.setParameter("ano", ano);
		if (mes != null) {
			query.setParameter("mes", mes);
		}
		
		List<SolicitacaoAlcada> solicitacoes = query.getResultList();
		
		float valor = 0;
		for (SolicitacaoAlcada solicitacaoAlcada : solicitacoes) {
			valor += solicitacaoAlcada.getValor().floatValue();
		}
		
		return valor;
	}

	@Override
	public SolicitacaoAlcada recuperaPeloIdentificador(Alcada alcada, String identificadorUnico) {
		SearchSeven search = new SearchSeven();

		search.addFilterEqual("alcada.identificador", alcada.getIdentificador());
		search.addFilterEqual("identificador", identificadorUnico);

		search.addFilterEmpty("dataInativo");
		
		search.addSort("id", true);

		search.setMaxResults(1);

		return this.searchUnique(search, SolicitacaoAlcada.class);
	}
	
}
