package br.com.centralit.api.model;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;

import br.com.centralit.api.framework.json.ViewsAlcada;
import br.com.centralit.api.framework.json.ViewsTabelasCorp;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.model.arquitetura.PersistentObject;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonView;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 14/07/2015 - 13:57:20
 *
 * @version 1.0.0
 *
 * @author lucas.ribeiro - (<a href="mailto:lucas.centralit@gmail.com">lucas.centralit@gmail.com</a>)
 *	
 */
@Entity
public class ResponsavelAlcada extends PersistentObject {

    /** Atributo serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** Atributo id. */
    @Id
	@GeneratedValue(strategy = GenerationType.AUTO)
    @JsonView({ Views.GenericView.class })
    private Long id;
    
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonIgnore
	private SolicitacaoAlcada solicitacaoAlcada;
	
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class, ViewsAlcada.RecuperacaoAlcadaView.class })
    private Colaborador colaborador;
    
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class, ViewsAlcada.RecuperacaoAlcadaView.class })
	private Usuario usuario;
	
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
    @JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class, ViewsAlcada.RecuperacaoAlcadaView.class })
    private NivelAutoridade nivelAutoridade;
	
    @Column(nullable = false)
    @JsonView({ Views.GenericView.class })
	protected Boolean delegacao;
    
    @ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JsonIgnore
	private CentroResultadoDelegacao centroResultadoDelegacao;
    
    @Column(nullable = false)
    @JsonView({ Views.GenericView.class })
	protected Boolean habilitado;
    
	@JsonView({ViewsTabelasCorp.SolicitacaoAlcadaView.class, ViewsAlcada.RecuperacaoAlcadaView.class} )
    @ManyToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
    @JoinTable(name="MotivoRejeicaoAlcada", 
               joinColumns=  @JoinColumn( name = "autorizadorAlcada_id"), 
               inverseJoinColumns= @JoinColumn(name = "motivoRejeicao_id") )
    private List<Dominio> motivosRejeicao;

    public ResponsavelAlcada() {
    	
    }
    
	public ResponsavelAlcada(SolicitacaoAlcada solicitacaoAlcada, Colaborador colaborador, NivelAutoridade nivelAutoridade, CentroResultadoDelegacao centroResultadoDelegacao) {
		this.solicitacaoAlcada = solicitacaoAlcada;
		this.habilitado = true;
		this.colaborador = colaborador;
		this.nivelAutoridade = nivelAutoridade;
		this.usuario = colaborador.getPessoa().getUsuario();
		this.centroResultadoDelegacao = centroResultadoDelegacao;
		this.setDelegacao(centroResultadoDelegacao != null);
	}

    /**
	 * Retorna o valor do atributo <code>solicitacaoAlcada</code>
	 *
	 * @return <code>SolicitacaoAlcada</code>
	 */
	public SolicitacaoAlcada getSolicitacaoAlcada() {
	
		return solicitacaoAlcada;
	}

	
	/**
	 * Define o valor do atributo <code>solicitacaoAlcada</code>.
	 *
	 * @param solicitacaoAlcada 
	 */
	public void setSolicitacaoAlcada(SolicitacaoAlcada solicitacaoAlcada) {
	
		this.solicitacaoAlcada = solicitacaoAlcada;
	}

	
	/**
	 * Retorna o valor do atributo <code>colaborador</code>
	 *
	 * @return <code>Colaborador</code>
	 */
	public Colaborador getColaborador() {
	
		return colaborador;
	}

	
	/**
	 * Define o valor do atributo <code>colaborador</code>.
	 *
	 * @param colaborador 
	 */
	public void setColaborador(Colaborador colaborador) {
	
		this.colaborador = colaborador;
	}

	
	/**
	 * Retorna o valor do atributo <code>usuario</code>
	 *
	 * @return <code>Usuario</code>
	 */
	public Usuario getUsuario() {
	
		return usuario;
	}

	
	/**
	 * Define o valor do atributo <code>usuario</code>.
	 *
	 * @param usuario 
	 */
	public void setUsuario(Usuario usuario) {
	
		this.usuario = usuario;
	}

	
	/**
	 * Retorna o valor do atributo <code>nivelAutoridade</code>
	 *
	 * @return <code>NivelAutoridade</code>
	 */
	public NivelAutoridade getNivelAutoridade() {
	
		return nivelAutoridade;
	}

	
	/**
	 * Define o valor do atributo <code>nivelAutoridade</code>.
	 *
	 * @param nivelAutoridade 
	 */
	public void setNivelAutoridade(NivelAutoridade nivelAutoridade) {
	
		this.nivelAutoridade = nivelAutoridade;
	}

	
	/**
	 * Retorna o valor do atributo <code>delegacao</code>
	 *
	 * @return <code>Boolean</code>
	 */
	public Boolean getDelegacao() {
	
		return delegacao;
	}

	
	/**
	 * Define o valor do atributo <code>delegacao</code>.
	 *
	 * @param delegacao 
	 */
	public void setDelegacao(Boolean delegacao) {
	
		this.delegacao = delegacao;
	}

	
	/**
	 * Retorna o valor do atributo <code>centroResultadoDelegacao</code>
	 *
	 * @return <code>CentroResultadoDelegacao</code>
	 */
	public CentroResultadoDelegacao getCentroResultadoDelegacao() {
	
		return centroResultadoDelegacao;
	}

	
	/**
	 * Define o valor do atributo <code>centroResultadoDelegacao</code>.
	 *
	 * @param centroResultadoDelegacao 
	 */
	public void setCentroResultadoDelegacao(CentroResultadoDelegacao centroResultadoDelegacao) {
	
		this.centroResultadoDelegacao = centroResultadoDelegacao;
	}

	
	/**
	 * Retorna o valor do atributo <code>habilitado</code>
	 *
	 * @return <code>Boolean</code>
	 */
	public Boolean getHabilitado() {
	
		return habilitado;
	}

	
	/**
	 * Define o valor do atributo <code>habilitado</code>.
	 *
	 * @param habilitado 
	 */
	public void setHabilitado(Boolean habilitado) {
	
		this.habilitado = habilitado;
	}

	
	/**
	 * Retorna o valor do atributo <code>motivosRejeicao</code>
	 *
	 * @return <code>List<Dominio></code>
	 */
	public List<Dominio> getMotivosRejeicao() {
	
		return motivosRejeicao;
	}

	
	/**
	 * Define o valor do atributo <code>motivosRejeicao</code>.
	 *
	 * @param motivosRejeicao 
	 */
	public void setMotivosRejeicao(List<Dominio> motivosRejeicao) {
	
		this.motivosRejeicao = motivosRejeicao;
	}

	
	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id 
	 */
	public void setId(Long id) {
	
		this.id = id;
	}

	@Override
	public Long getId() {
		return this.id;
	}

	public void adicionaMotivoRejeicao(Dominio motivoRejeicao) {
		if (this.getMotivosRejeicao() == null) {
			this.setMotivosRejeicao(new ArrayList<Dominio>());
		}
		
		this.setHabilitado(false);
		this.getMotivosRejeicao().add(motivoRejeicao);
	}



}
