package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.PrePersist;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import br.com.centralit.api.framework.json.ViewsAlcada;
import br.com.centralit.api.framework.json.ViewsTabelasCorp;
import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAudit;
import br.com.centralit.framework.util.UtilDate;

import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 14/07/2015 - 13:57:20
 *
 * @version 1.0.0
 *
 * @author lucas.ribeiro - (<a href="mailto:lucas.centralit@gmail.com">lucas.centralit@gmail.com</a>)
 *	
 */
@Entity
public class SolicitacaoAlcada extends PersistentObjectAudit {

    /** Atributo serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** Atributo id. */
    @Id
	@GeneratedValue(strategy = GenerationType.AUTO)
    @JsonView({ Views.GenericView.class })
    private Long id;
    
	@JsonView({ Views.GenericView.class })
    private Integer ano;

	@JsonView({ Views.GenericView.class })
    private Integer mes;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class })
	private Alcada alcada;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class })
	private CentroResultado centroResultado;

	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class })
	private Usuario solicitante;

	@Column(nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
    @JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
    @JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
    @JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class })
    private Calendar dataReferencia;
    
    @Column(nullable = false)
	@JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class })
	protected String requisicaoOrigem;
	
	@JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class })
	protected String identificador;

    @ManyToOne(fetch = FetchType.LAZY, optional = true)
    @JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class })
    private Dominio tipoUtilizacao;
    
    @Column(nullable = false, precision = 20, scale = 2)
    @JsonDeserialize(using = MoneyDeserializer.class)
    @JsonView({ Views.GenericView.class })
    private BigDecimal valor;
    
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class, ViewsAlcada.RecuperacaoAlcadaView.class })
    private Dominio situacao;
    
    @ManyToOne(fetch = FetchType.LAZY, optional = true)
    @JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class })
    private ResponsavelAlcada aprovador;

    @OneToMany(fetch = FetchType.LAZY, mappedBy = "solicitacaoAlcada", cascade = CascadeType.ALL)
	@JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class, ViewsAlcada.RecuperacaoAlcadaView.class })
	private List<ResponsavelAlcada> responsaveis;
    
    @Transient
    @JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class, ViewsAlcada.RecuperacaoAlcadaView.class })
    private String usernameAprovador;

    @Transient
    @JsonView({ ViewsTabelasCorp.SolicitacaoAlcadaView.class })
    private Usuario usuarioAprovador;

    @Override
	public Long getId() {
		return this.id;
	}

	public SolicitacaoAlcada() {
		
	}
	
	public SolicitacaoAlcada( final Alcada alcada, final CentroResultado centroResultado, final Usuario solicitante
							, final Calendar dataReferencia, String requisicaoOrigem, Dominio tipoUtilizacao, BigDecimal valor) {
		this.alcada = alcada;
		this.centroResultado = centroResultado;
		this.solicitante = solicitante;
		this.dataReferencia = dataReferencia;
		this.requisicaoOrigem = requisicaoOrigem;
		this.tipoUtilizacao = tipoUtilizacao;
		this.valor = valor;
	}

	/**
	 * Retorna o valor do atributo <code>alcada</code>
	 *
	 * @return <code>Alcada</code>
	 */
	public Alcada getAlcada() {
	
		return alcada;
	}

	
	/**
	 * Define o valor do atributo <code>alcada</code>.
	 *
	 * @param alcada 
	 */
	public void setAlcada(Alcada alcada) {
	
		this.alcada = alcada;
	}

	
	/**
	 * Retorna o valor do atributo <code>centroResultado</code>
	 *
	 * @return <code>CentroResultado</code>
	 */
	public CentroResultado getCentroResultado() {
	
		return centroResultado;
	}

	
	/**
	 * Define o valor do atributo <code>centroResultado</code>.
	 *
	 * @param centroResultado 
	 */
	public void setCentroResultado(CentroResultado centroResultado) {
	
		this.centroResultado = centroResultado;
	}

	
	/**
	 * Retorna o valor do atributo <code>solicitante</code>
	 *
	 * @return <code>Usuario</code>
	 */
	public Usuario getSolicitante() {
	
		return solicitante;
	}

	
	/**
	 * Define o valor do atributo <code>solicitante</code>.
	 *
	 * @param solicitante 
	 */
	public void setSolicitante(Usuario solicitante) {
	
		this.solicitante = solicitante;
	}

	
	/**
	 * Retorna o valor do atributo <code>dataReferencia</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataReferencia() {
	
		return dataReferencia;
	}

	
	/**
	 * Define o valor do atributo <code>dataReferencia</code>.
	 *
	 * @param dataReferencia 
	 */
	public void setDataReferencia(Calendar dataReferencia) {
	
		this.dataReferencia = dataReferencia;
	}

	
	/**
	 * Retorna o valor do atributo <code>requisicaoOrigem</code>
	 *
	 * @return <code>String</code>
	 */
	public String getRequisicaoOrigem() {
	
		return requisicaoOrigem;
	}

	
	/**
	 * Define o valor do atributo <code>requisicaoOrigem</code>.
	 *
	 * @param requisicaoOrigem 
	 */
	public void setRequisicaoOrigem(String requisicaoOrigem) {
	
		this.requisicaoOrigem = requisicaoOrigem;
	}

	
	/**
	 * Retorna o valor do atributo <code>tipoUtilizacao</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getTipoUtilizacao() {
	
		return tipoUtilizacao;
	}

	
	/**
	 * Define o valor do atributo <code>tipoUtilizacao</code>.
	 *
	 * @param tipoUtilizacao 
	 */
	public void setTipoUtilizacao(Dominio tipoUtilizacao) {
	
		this.tipoUtilizacao = tipoUtilizacao;
	}

	
	/**
	 * Retorna o valor do atributo <code>valor</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValor() {
	
		return valor;
	}

	
	/**
	 * Define o valor do atributo <code>valor</code>.
	 *
	 * @param valor 
	 */
	public void setValor(BigDecimal valor) {
	
		this.valor = valor;
	}

	
	/**
	 * Retorna o valor do atributo <code>situacao</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getSituacao() {
	
		return situacao;
	}

	
	/**
	 * Define o valor do atributo <code>situacao</code>.
	 *
	 * @param situacao 
	 */
	public void setSituacao(Dominio situacao) {
	
		this.situacao = situacao;
	}

	
	/**
	 * Retorna o valor do atributo <code>aprovador</code>
	 *
	 * @return <code>ResponsavelAlcada</code>
	 */
	public ResponsavelAlcada getAprovador() {
	
		return aprovador;
	}

	
	/**
	 * Retorna o valor do atributo <code>responsaveis</code>
	 *
	 * @return <code>Collection<ResponsavelAlcada></code>
	 */
	public List<ResponsavelAlcada> getResponsaveis() {
	
		return responsaveis;
	}

	
	/**
	 * Define o valor do atributo <code>responsaveis</code>.
	 *
	 * @param responsaveis 
	 */
	public void setResponsaveis(List<ResponsavelAlcada> responsaveis) {
	
		this.responsaveis = responsaveis;
	}

	
	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id 
	 */
	public void setId(Long id) {
	
		this.id = id;
	}


	
	/**
	 * Retorna o valor do atributo <code>ano</code>
	 *
	 * @return <code>Integer</code>
	 */
	public Integer getAno() {
	
		return ano;
	}


	
	/**
	 * Define o valor do atributo <code>ano</code>.
	 *
	 * @param ano 
	 */
	public void setAno(Integer ano) {
	
		this.ano = ano;
	}


	
	/**
	 * Retorna o valor do atributo <code>mes</code>
	 *
	 * @return <code>Integer</code>
	 */
	public Integer getMes() {
	
		return mes;
	}


	
	/**
	 * Define o valor do atributo <code>mes</code>.
	 *
	 * @param mes 
	 */
	public void setMes(Integer mes) {
	
		this.mes = mes;
	}
	
	@PrePersist
	protected void onPrePersist() {
		super.onPrePersist();
		if(this.dataReferencia != null){
			int ano = new Integer(UtilDate.getAno(this.getDataReferencia().getTime()));
			int mes = new Integer(UtilDate.getMes(this.getDataReferencia().getTime()));
			
			this.setMes(mes);
			this.setAno(ano);
		}
	}

	
	/**
	 * Retorna o valor do atributo <code>usernameAprovador</code>
	 *
	 * @return <code>String</code>
	 */
	public String getUsernameAprovador() {
	
		return usernameAprovador;
	}

	
	/**
	 * Define o valor do atributo <code>usernameAprovador</code>.
	 *
	 * @param usernameAprovador 
	 */
	public void setUsernameAprovador(String usernameAprovador) {
	
		this.usernameAprovador = usernameAprovador;
	}

	
	/**
	 * Retorna o valor do atributo <code>usuarioAprovador</code>
	 *
	 * @return <code>Usuario</code>
	 */
	public Usuario getUsuarioAprovador() {
	
		return usuarioAprovador;
	}

	
	/**
	 * Define o valor do atributo <code>usuarioAprovador</code>.
	 *
	 * @param usuarioAprovador 
	 */
	public void setUsuarioAprovador(Usuario usuarioAprovador) {
	
		this.usuarioAprovador = usuarioAprovador;
	}

	
	/**
	 * Retorna o valor do atributo <code>identificador</code>
	 *
	 * @return <code>String</code>
	 */
	public String getIdentificador() {
	
		return identificador;
	}

	
	/**
	 * Define o valor do atributo <code>identificador</code>.
	 *
	 * @param identificador 
	 */
	public void setIdentificador(String identificador) {
	
		this.identificador = identificador;
	}

	
	/**
	 * Define o valor do atributo <code>aprovador</code>.
	 *
	 * @param aprovador 
	 */
	public void setAprovador(ResponsavelAlcada aprovador) {
	
		this.aprovador = aprovador;
	}

	
}
