package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.CentroResultadoDelegacaoDao;
import br.com.centralit.api.model.Alcada;
import br.com.centralit.api.model.CentroResultadoDelegacao;
import br.com.centralit.api.model.CentroResultadoDelegacaoRequisicao;
import br.com.centralit.api.model.Colaborador;
import br.com.centralit.api.service.CentroResultadoDelegacaoService;
import br.com.centralit.api.service.CentroResultadoResponsavelAlcadaService;
import br.com.centralit.api.service.ColaboradorService;
import br.com.centralit.framework.esi.enumerated.VariableTypeEnum;
import br.com.centralit.framework.esi.environment.EnvironmentVariable;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;
import br.com.centralit.framework.util.UtilString;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 30/06/2015 - 14:10:09
 * 
 * @version 1.0.0
 * 
 * @author lucas.ribeiro - (<a href="mailto:lucas.ribeiro@centralit.com.br">lucas.ribeiro@centralit.com.br</a>)
 * 
 */
@Service("centroResultadoDelegacaoService")
public class CentroResultadoDelegacaoServiceImpl extends GenericServiceImpl<CentroResultadoDelegacao, Long> implements CentroResultadoDelegacaoService {

	/** Atributo centroResultadoResponsavelAlcadaService. */
	@Autowired
	private CentroResultadoResponsavelAlcadaService centroResultadoResponsavelAlcadaService;

	@Autowired
	private ColaboradorService colaboradorService;

	private CentroResultadoDelegacaoDao centroResultadoDelegacaoDao;

	private void disparaFluxoDelegacao(CentroResultadoDelegacao centroResultadoDelegacao) {

		Alcada alcada = centroResultadoDelegacao.getCentroResultadoResponsavelAlcada().getAlcada();
		if (!UtilString.isNullOrEmpty(alcada.getFluxoDelegacao())) {
			Usuario usuario = centroResultadoDelegacao.getCentroResultadoResponsavelAlcada().getCentroResultadoResponsavel().getResponsavel().getPessoa().getUsuario();
			if (!UtilObjeto.isReferencia(usuario)) {
				throw new BusinessException("MSG.USUARIO_NAO_ASSOCIADO_RESPONSAVEL", CodigoErro.REGRA_NEGOCIO.getValue());
			}
			String responsavel = usuario.getUsername();

			Colaborador colaborador = colaboradorService.getReference(centroResultadoDelegacao.getDelegado().getId());
			usuario = colaborador.getPessoa().getUsuario();
			if (!UtilObjeto.isReferencia(usuario)) {
				throw new BusinessException("MSG.USUARIO_NAO_ASSOCIADO_DELEGACAO", CodigoErro.REGRA_NEGOCIO.getValue());
			}
			String delegado = usuario.getUsername();

			String abrangencia = centroResultadoDelegacao.getDominioTipoAbrangenciaDelegacao().getNome();

			List<EnvironmentVariable> variaveis = new ArrayList<EnvironmentVariable>();
			variaveis.add(new EnvironmentVariable("alcada", VariableTypeEnum.TEXT, alcada.getIdentificador(), false));
			variaveis.add(new EnvironmentVariable("responsavel", VariableTypeEnum.TEXT, responsavel, false));
			variaveis.add(new EnvironmentVariable("delegado", VariableTypeEnum.TEXT, delegado, false));
			variaveis.add(new EnvironmentVariable("abrangencia", VariableTypeEnum.TEXT, abrangencia, false));
			variaveis.add(new EnvironmentVariable("requisicoes", VariableTypeEnum.JAVAOBJECT, centroResultadoDelegacao.getRequisicoesStr(), false));

			this.startProcess(alcada.getFluxoDelegacao(), variaveis);
		}
	}

	@Autowired
	public CentroResultadoDelegacaoServiceImpl( CentroResultadoDelegacaoDao centroResultadoDelegacaoDao, @Qualifier("centroResultadoDelegacaoValidator") Validator validator ) {

		this.dao = centroResultadoDelegacaoDao;
		this.centroResultadoDelegacaoDao = centroResultadoDelegacaoDao;
		this.validator = validator;
	}

	@Override
	public CentroResultadoDelegacao save(CentroResultadoDelegacao centroResultadoDelegacao) {

		// VALIDA CAMPO OBRIGATÓRIO DA ENTIDADE
		if (UtilObjeto.isReferencia(this.validator)) {
			this.validarEntidade(centroResultadoDelegacao, this.validator);
		}

		this.existeDelegacaoParaDelegadoNaAlcadaENoPeriodoSeleciona(centroResultadoDelegacao);

		montaCentroResultadoDelegacao(centroResultadoDelegacao);

		// SALVA A TRANSFERÊNCIA
		CentroResultadoDelegacao entitySaved = (CentroResultadoDelegacao) this.centroResultadoDelegacaoDao.save(centroResultadoDelegacao);

		this.disparaFluxoDelegacao(entitySaved);

		return entitySaved;
	}

	@Override
	public CentroResultadoDelegacao merge(CentroResultadoDelegacao entity) {

		return super.merge(entity);
	}

	@Override
	public List<CentroResultadoDelegacao> saveList(List<CentroResultadoDelegacao> centroResultadoDelegacaoList) {

		if (!UtilColecao.isVazio(centroResultadoDelegacaoList)) {
			for (CentroResultadoDelegacao centroResultadoDelegacao : centroResultadoDelegacaoList) {
				
				this.save(centroResultadoDelegacao);
				
			}

			return super.saveList(centroResultadoDelegacaoList);
		}

		return new ArrayList<CentroResultadoDelegacao>();
	}

	private void existeDelegacaoParaDelegadoNaAlcadaENoPeriodoSeleciona(CentroResultadoDelegacao centroResultadoDelegacao) {

		if (this.centroResultadoDelegacaoDao.existeDelegacaoParaDelegadoNaAlcadaENoPeriodoSeleciona(centroResultadoDelegacao)) {
			throw new BusinessException("MSG.EXISTE_DELEGACAO_RESPONSAVEL_ALCADA_SELECIONADA", CodigoErro.REGRA_NEGOCIO.getValue());
		}
	}

	private void montaCentroResultadoDelegacao(CentroResultadoDelegacao centroResultadoDelegacao) {

		// MONTA OS DADOS DO OBJETO CENTRORESULTADODELEGACAO
		centroResultadoDelegacao.setCentroResultadoResponsavelAlcada(this.centroResultadoResponsavelAlcadaService.find(centroResultadoDelegacao.getCentroResultadoResponsavelAlcada().getId()));

		this.resolveReferenciaCentroResultadoDelegacaoInCentroResultadoDelegacaoRequisicoes(centroResultadoDelegacao);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author renato.jesus
	 * 
	 * @param centroResultadoDelegacao
	 */
	private void resolveReferenciaCentroResultadoDelegacaoInCentroResultadoDelegacaoRequisicoes(CentroResultadoDelegacao centroResultadoDelegacao) {

		if (!UtilColecao.isVazio(centroResultadoDelegacao.getRequisicoes())) {
			for (CentroResultadoDelegacaoRequisicao centroResultadoDelegacaoRequisicao : centroResultadoDelegacao.getRequisicoes()) {
				centroResultadoDelegacaoRequisicao.setCentroResultadoDelegacao(centroResultadoDelegacao);
			}
		}
	}

	@Override
	public List<CentroResultadoDelegacao> buscaHistoricoDelegacoes(Long idCentroResultado) {

		return centroResultadoDelegacaoDao.buscaHistoricoDelegacoes(idCentroResultado);
	}

	@Override
	public List<CentroResultadoDelegacao> buscaHistoricoDelegacoesUsuario(Long idUsuario) {

		return centroResultadoDelegacaoDao.buscaHistoricoDelegacoesUsuario(idUsuario);
	}

	@Override
	public CentroResultadoDelegacao revogar(Long idCentroResultadoDelegacao) {

		CentroResultadoDelegacao centroResultadoDelegacao = this.find(idCentroResultadoDelegacao);
		centroResultadoDelegacao.setRevogada(true);
		centroResultadoDelegacao.setResponsavelRevogacao(this.getUsuario());
		centroResultadoDelegacao.setDataRevogacao(UtilDate.getDataAtualCalendar());

		CentroResultadoDelegacao entityUpdated = merge(centroResultadoDelegacao);
		return entityUpdated;
	}

}
