package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.SolicitacaoAlcadaDao;
import br.com.centralit.api.model.Alcada;
import br.com.centralit.api.model.AlcadaNivelAutoridade;
import br.com.centralit.api.model.CentroResultado;
import br.com.centralit.api.model.CentroResultadoDelegacao;
import br.com.centralit.api.model.CentroResultadoDelegacaoRequisicao;
import br.com.centralit.api.model.CentroResultadoResponsavel;
import br.com.centralit.api.model.CentroResultadoResponsavelAlcada;
import br.com.centralit.api.model.Colaborador;
import br.com.centralit.api.model.LimiteAprovacao;
import br.com.centralit.api.model.LimiteAprovacaoAlcada;
import br.com.centralit.api.model.LimiteAprovacaoNivelAutoridade;
import br.com.centralit.api.model.LimiteAprovacaoValor;
import br.com.centralit.api.model.NivelAutoridade;
import br.com.centralit.api.model.NivelAutoridadeGrupo;
import br.com.centralit.api.model.ResponsavelAlcada;
import br.com.centralit.api.model.SolicitacaoAlcada;
import br.com.centralit.api.model.ValorAcumulado;
import br.com.centralit.api.service.AlcadaService;
import br.com.centralit.api.service.CentroResultadoService;
import br.com.centralit.api.service.ColaboradorService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.GrupoUsuarioService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.LimiteAprovacaoAlcadaService;
import br.com.centralit.api.service.NivelAutoridadeService;
import br.com.centralit.api.service.PessoaService;
import br.com.centralit.api.service.SolicitacaoAlcadaService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.GrupoUsuario;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;
import br.com.centralit.framework.util.UtilString;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 18/06/2015 - 16:57:02
 * 
 * @version 1.0.0
 * 
 * @author lucas.ribeiro - (<a href="mailto:lucas.centralit@gmail.com">lucas.centralit@gmail.com</a>)
 * 
 */
@Service("solicitacaoAlcadaService")
public class SolicitacaoAlcadaServiceImpl extends GenericServiceImpl<SolicitacaoAlcada, Long> implements SolicitacaoAlcadaService {

	public static final Long FAIXA_VALOR = 1l;

	public static final Long QUALQUER_VALOR = 2l;

	public static final Long NAO_APLICA = 3l;

	public static final Long TODOS = 2l;

	public static final Long INDIVIDUAL = 1l;

	public static final Long MENSAL = 2l;

	public static final Long ANUAL = 3l;

	public static final Long LIMITE_VALOR = 1L;

	public static final Long HIERARQUIA_SOLICITANTE = 2L;

	public static final Long PERMISSAO_AUTORIDADE = 3L;

	public static final Long REGRAS_PROCESSO = 4L;

	public static final Long USUARIO_NAO_EXISTE = 5L;

	public static final Long PERTENCE_GRUPO_RESPONSAVEL = 6L;

	public static final Long GRUPO_NAO_ASSOCIADO = 7L;

	public static final Long HIERARQUIA_AUTORIDADE = 8L;

	public static final String SITUACAO_GERADA = "GERADA";

	public static final String SITUACAO_APROVADA = "APROVADA";

	public static final String SITUACAO_CANCELADA = "CANCELADA";

	@Autowired
	private AlcadaService alcadaService;

	@Autowired
	private DominioService dominioService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private LimiteAprovacaoAlcadaService limiteAprovacaoAlcadaService;

	@Autowired
	private CentroResultadoService centroResultadoService;

	@Autowired
	private GrupoUsuarioService grupoUsuarioService;

	@Autowired
	private ColaboradorService colaboradorService;

	@Autowired
	private PessoaService pessoaService;

	@Autowired
	private NivelAutoridadeService nivelAutoridadeService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	private SolicitacaoAlcadaDao solicitacaoAlcadaDao;

	private Dominio idioma = null;

	@Autowired
	public SolicitacaoAlcadaServiceImpl( SolicitacaoAlcadaDao solicitacaoAlcadaDao) {

		this.dao = solicitacaoAlcadaDao;
		this.solicitacaoAlcadaDao = solicitacaoAlcadaDao;
	}

	@Override
	public SolicitacaoAlcada save(SolicitacaoAlcada solicitacaoAlcada) {

		if (solicitacaoAlcada.getIdentificador() != null) {
			SolicitacaoAlcada solicitacaoAlcadaRef = this.recuperaPeloIdentificador(solicitacaoAlcada.getAlcada(), solicitacaoAlcada.getIdentificador());
			if (UtilObjeto.isReferencia(solicitacaoAlcadaRef)) {
				throw new BusinessException(internacionalizacaoService.getTranslate("MSG.SOLICITACAO_ALCADA_JA_EXISTE", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
			}
		}

		solicitacaoAlcada.setSituacao(dominioService.findByChaveAndNome("situacaoSolicitacaoAlcada", SITUACAO_GERADA));

		List<ResponsavelAlcada> responsaveis = this.getResponsaveis(solicitacaoAlcada);

		solicitacaoAlcada.setResponsaveis(responsaveis);

		this.verificaAprovacao(solicitacaoAlcada);

		solicitacaoAlcada = super.save(solicitacaoAlcada);

		return solicitacaoAlcada;
	}

	@Override
	public boolean remove(SolicitacaoAlcada solicitacaoAlcada) {

		solicitacaoAlcada = this.getReference(solicitacaoAlcada.getId());

		if (solicitacaoAlcada.getSituacao().getNome().equals(SITUACAO_APROVADA)) {
			throw new BusinessException(internacionalizacaoService.getTranslate("MSG.SOLICITACAO_NAO_PERMITE_EXCLUSAO", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
		}

		return super.remove(solicitacaoAlcada);
	}

	@Override
	public SolicitacaoAlcada aprovaSolicitacao(SolicitacaoAlcada solicitacaoAlcada, Usuario aprovador) {

		if (!UtilObjeto.isReferencia(solicitacaoAlcada)) {
			throw new BusinessException(internacionalizacaoService.getTranslate("MSG.SOLICITACAO_ALCADA_NAO_ENCONTRADA", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
		}

		if (solicitacaoAlcada.getSituacao().getNome().equals(SITUACAO_APROVADA)) {
			throw new BusinessException(internacionalizacaoService.getTranslate("MSG.SOLICITACAO_NAO_PERMITE_AUTORIZACAO", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
		}

		ResponsavelAlcada responsavel = null;
		for (ResponsavelAlcada responsavelAlcada : solicitacaoAlcada.getResponsaveis()) {
			if (responsavelAlcada.getHabilitado() && responsavelAlcada.getUsuario().getUsername().equalsIgnoreCase(aprovador.getUsername())) {
				responsavel = responsavelAlcada;
				break;
			}
		}

		if (responsavel == null) {
			throw new BusinessException(internacionalizacaoService.getTranslate("MSG.APROVADOR_NAO_ENCONTRADO", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
		}

		solicitacaoAlcada.setAprovador(responsavel);
		solicitacaoAlcada.setSituacao(dominioService.findByChaveAndNome("situacaoSolicitacaoAlcada", SITUACAO_APROVADA));

		return this.merge(solicitacaoAlcada);
	}

	@Override
	public SolicitacaoAlcada aprovaPeloIdentificador(SolicitacaoAlcada solicitacaoAlcada) {

		String usernameAprovador = solicitacaoAlcada.getUsernameAprovador();
		if (UtilString.isNullOrEmpty(solicitacaoAlcada.getUsernameAprovador())) {
			throw new BusinessException(internacionalizacaoService.getTranslate("MSG.APROVADOR_NAO_ENCONTRADO", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
		}

		Usuario aprovador = usuarioService.getUserByUsername(usernameAprovador);
		if (!UtilObjeto.isReferencia(aprovador)) {
			throw new BusinessException(internacionalizacaoService.getTranslate("MSG.APROVADOR_NAO_ENCONTRADO", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
		}

		SolicitacaoAlcada solicitacaoAlcadaRef = this.recuperaPeloIdentificador(solicitacaoAlcada.getAlcada(), solicitacaoAlcada.getIdentificador());
		if (!UtilObjeto.isReferencia(solicitacaoAlcadaRef)) {
			throw new BusinessException(internacionalizacaoService.getTranslate("MSG.SOLICITACAO_ALCADA_NAO_ENCONTRADA", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
		}

		if (solicitacaoAlcada.getValor() != null && solicitacaoAlcada.getValor().doubleValue() > 0.0) {
			solicitacaoAlcadaRef.setValor(solicitacaoAlcada.getValor());
		}

		return this.aprovaSolicitacao(solicitacaoAlcadaRef, aprovador);
	}

	@Override
	public SolicitacaoAlcada cancelaAprovacaoPeloIdentificador(SolicitacaoAlcada solicitacaoAlcada) {

		SolicitacaoAlcada solicitacaoAlcadaRef = this.recuperaPeloIdentificador(solicitacaoAlcada.getAlcada(), solicitacaoAlcada.getIdentificador());
		if (!UtilObjeto.isReferencia(solicitacaoAlcadaRef)) {
			throw new BusinessException(internacionalizacaoService.getTranslate("MSG.SOLICITACAO_ALCADA_NAO_ENCONTRADA", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
		}

		return this.cancelaAprovacao(solicitacaoAlcadaRef);
	}

	@Override
	public SolicitacaoAlcada cancelaAprovacao(SolicitacaoAlcada solicitacaoAlcada) {

		if (!UtilObjeto.isReferencia(solicitacaoAlcada)) {
			throw new BusinessException(internacionalizacaoService.getTranslate("MSG.SOLICITACAO_ALCADA_NAO_ENCONTRADA", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
		}

		if (solicitacaoAlcada.getSituacao().getNome().equals(SITUACAO_APROVADA)) {
			solicitacaoAlcada.setAprovador(null);
			solicitacaoAlcada.setSituacao(dominioService.findByChaveAndNome("situacaoSolicitacaoAlcada", SITUACAO_CANCELADA));
			solicitacaoAlcada.setDataInativo(UtilDate.getDataAtualCalendar());
			solicitacaoAlcada = this.merge(solicitacaoAlcada);
		}

		return solicitacaoAlcada;
	}

	@Override
	public List<ResponsavelAlcada> getResponsaveis(SolicitacaoAlcada solicitacaoAlcada) {

		this.validaSolicitacao(solicitacaoAlcada);

		List<ResponsavelAlcada> responsaveis = this.getResponsaveisCentroResultado(solicitacaoAlcada);

		List<LimiteAprovacao> limitesAprovacao = this.getLimitesAprovacao(solicitacaoAlcada);

		this.validaResponsaveis(solicitacaoAlcada, limitesAprovacao, responsaveis);

		this.trataAlcadasNivelSuperior(solicitacaoAlcada, limitesAprovacao, responsaveis);

		return responsaveis;
	}

	@Override
	public SolicitacaoAlcada recuperaSolicitacao(Alcada alcada, String identificadorUnico) {

		SolicitacaoAlcada solicitacaoAlcada = solicitacaoAlcadaDao.recuperaPeloIdentificador(alcada, identificadorUnico);
		return recuperaRelacionamentos(solicitacaoAlcada);
	}

	protected SolicitacaoAlcada recuperaRelacionamentos(SolicitacaoAlcada solicitacaoAlcada) {

		if (UtilObjeto.isReferencia(solicitacaoAlcada) && !solicitacaoAlcada.isNew()) {
			solicitacaoAlcada.setSituacao(dominioService.find(solicitacaoAlcada.getSituacao().getId()));

			if (solicitacaoAlcada.getAprovador() != null) {
				solicitacaoAlcada.setUsernameAprovador(solicitacaoAlcada.getAprovador().getUsuario().getUsername());
			}
			for (ResponsavelAlcada responsavelAlcada : solicitacaoAlcada.getResponsaveis()) {
				// finds necessários para resolver problemas na serialização
				Colaborador colaborador = colaboradorService.find(responsavelAlcada.getColaborador().getId());
				colaborador.setPessoa(pessoaService.find(colaborador.getPessoa().getId()));
				responsavelAlcada.setColaborador(colaborador);
				responsavelAlcada.setUsuario(usuarioService.find(responsavelAlcada.getUsuario().getId()));
				responsavelAlcada.setNivelAutoridade(nivelAutoridadeService.find(responsavelAlcada.getNivelAutoridade().getId()));

				if (responsavelAlcada.getMotivosRejeicao() != null) {
					for (Dominio motivo : responsavelAlcada.getMotivosRejeicao()) {
						motivo = dominioService.find(motivo.getId());
					}
				}
			}
		}
		return solicitacaoAlcada;
	}

	protected SolicitacaoAlcada recuperaPeloIdentificador(Alcada alcada, String identificadorUnico) {

		return solicitacaoAlcadaDao.recuperaPeloIdentificador(alcada, identificadorUnico);
	}

	protected void verificaAprovacao(SolicitacaoAlcada solicitacaoAlcada) {

		if (UtilObjeto.isReferencia(solicitacaoAlcada.getUsuarioAprovador())) {
			List<ResponsavelAlcada> responsaveis = solicitacaoAlcada.getResponsaveis();

			// Verifica se aprovador está na lista dos responsáveis
			Usuario aprovador = solicitacaoAlcada.getUsuarioAprovador();
			ResponsavelAlcada responsavelAprovacao = null;
			for (ResponsavelAlcada responsavelAlcada : responsaveis) {
				if (responsavelAlcada.getUsuario().getUsername().equalsIgnoreCase(aprovador.getUsername())) {
					responsavelAprovacao = responsavelAlcada;
					break;
				}
			}

			if (responsavelAprovacao == null) {
				// Cria aprovador, se não estiver na lista de responsáveis
				Colaborador colaborador = colaboradorService.findPorUsuario(aprovador);
				if (colaborador != null && !this.existeResponsavel(colaborador, responsaveis)) {
					responsavelAprovacao = new ResponsavelAlcada(solicitacaoAlcada, colaborador, null, null);
					this.atribuiNivelAutoridade(solicitacaoAlcada.getAlcada(), responsavelAprovacao);
					responsaveis.add(responsavelAprovacao);
				}
			}

			if (responsavelAprovacao == null) {
				throw new BusinessException(internacionalizacaoService.getTranslate("MSG.APROVADOR_NAO_ENCONTRADO", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
			}

			solicitacaoAlcada.setAprovador(responsavelAprovacao);
			solicitacaoAlcada.setSituacao(dominioService.findByChaveAndNome("situacaoSolicitacaoAlcada", SITUACAO_APROVADA));
		}
	}

	protected Dominio getIdioma() {

		if (this.idioma == null) {
			this.idioma = this.dominioService.findByChaveAndCodigo("tipoIdioma", Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		}
		return this.idioma;
	}

	protected void trataAlcadasNivelSuperior(SolicitacaoAlcada solicitacaoAlcada, List<LimiteAprovacao> limitesAprovacao, List<ResponsavelAlcada> responsaveis) {

		int i = 0;
		for (ResponsavelAlcada responsavelAlcada : responsaveis) {
			if (responsavelAlcada.getHabilitado())
				i++;
		}

		if (i == 0) {
			for (LimiteAprovacao limiteAprovacao : limitesAprovacao) {
				if (!limiteAprovacao.getDominioTipoLimite().getCodigo().equals(FAIXA_VALOR) && limiteAprovacao.getDominioTipoAbrangencia().getCodigo().equals(TODOS)) {
					for (LimiteAprovacaoNivelAutoridade limiteAprovacaoNivelAutoridade : limiteAprovacao.getNiveisAutoridades()) {
						for (NivelAutoridadeGrupo nivelAutoridadeGrupo : limiteAprovacaoNivelAutoridade.getNivelAutoridade().getGrupos()) {
							Collection<GrupoUsuario> gruposUsuario = grupoUsuarioService.findPorIdGrupo(nivelAutoridadeGrupo.getGrupo().getId());
							for (GrupoUsuario grupoUsuario : gruposUsuario) {
								Colaborador colaborador = colaboradorService.findPorUsuario(grupoUsuario.getUsuario());
								if (colaborador != null && !this.existeResponsavel(colaborador, responsaveis)) {
									responsaveis.add(new ResponsavelAlcada(solicitacaoAlcada, colaborador, nivelAutoridadeGrupo.getNivelAutoridade(), null));
								}
							}
						}
					}
				}
			}
		}
	}

	protected void recuperaAlcada(SolicitacaoAlcada solicitacaoAlcada) {

		Alcada alcada = solicitacaoAlcada.getAlcada();
		solicitacaoAlcada.setAlcada(null);

		if (alcada.getId() != null) {
			solicitacaoAlcada.setAlcada(alcadaService.getReference(alcada.getId()));
		} else if (alcada.getIdentificador() != null) {
			solicitacaoAlcada.setAlcada(alcadaService.findByIdentificaodr(alcada.getIdentificador()));
		}
	}

	protected void recuperaCentroResultado(SolicitacaoAlcada solicitacaoAlcada) {

		CentroResultado centroResultado = solicitacaoAlcada.getCentroResultado();
		CentroResultado centroResultadoAux = null;

		if (centroResultado.getId() != null) {
			centroResultadoAux = centroResultadoService.getReference(centroResultado.getId());
		} 
		if (centroResultadoAux == null && centroResultado.getCodigoSistemaOrigem() != null) {
			centroResultadoAux = centroResultadoService.findByCodigoSistemaOrigem(centroResultado.getCodigoSistemaOrigem());
		}
		if (centroResultadoAux == null && centroResultado.getCodigo() != null) {
			centroResultadoAux = centroResultadoService.findByCodigo(centroResultado.getCodigo());
			if (centroResultadoAux == null) {
				String codigo = centroResultado.getCodigo().replaceAll(".", "");
				centroResultadoAux = centroResultadoService.findByCodigoSistemaOrigem(codigo);
			}
		} 
		
		solicitacaoAlcada.setCentroResultado(centroResultadoAux);
	}

	protected void recuperaSolicitante(SolicitacaoAlcada solicitacaoAlcada) {

		Usuario solicitante = solicitacaoAlcada.getSolicitante();
		solicitacaoAlcada.setSolicitante(null);

		if (solicitante.getId() != null) {
			solicitacaoAlcada.setSolicitante(usuarioService.getReference(solicitante.getId()));
		} else if (solicitante.getUsername() != null) {
			solicitacaoAlcada.setSolicitante(usuarioService.getUserByUsername(solicitante.getUsername()));
		}
	}

	protected void recuperaTipoSolicitacao(SolicitacaoAlcada solicitacaoAlcada) {

		Dominio tipoUtilizacao = solicitacaoAlcada.getTipoUtilizacao();
		solicitacaoAlcada.setTipoUtilizacao(null);

		if (tipoUtilizacao.getId() != null) {
			solicitacaoAlcada.setTipoUtilizacao(dominioService.getReference(tipoUtilizacao.getId()));
		} else if (tipoUtilizacao.getNome() != null) {
			solicitacaoAlcada.setTipoUtilizacao(dominioService.findByChaveAndNome("tipoUtilizacao", tipoUtilizacao.getNome()));
		}
	}

	protected void validaAtributos(SolicitacaoAlcada solicitacaoAlcada) {

		String msg = ": "+internacionalizacaoService.getTranslate("LABEL.O_CAMPO_OBRIGATORIO", this.getIdioma());

		if (solicitacaoAlcada.getAlcada() == null) {
			throw new BusinessException(internacionalizacaoService.getTranslate("ALCADA.LABEL.ALCADA", this.getIdioma())+msg, CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue());
		}
		if (solicitacaoAlcada.getCentroResultado() == null) {
			throw new BusinessException(internacionalizacaoService.getTranslate("ALCADA.LABEL.CENTRO_RESULTADO", this.getIdioma())+msg, CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue());
		}
		if (solicitacaoAlcada.getValor() == null) {
			throw new BusinessException(internacionalizacaoService.getTranslate("SOLICITACAO_ALCADA.VALOR", this.getIdioma())+msg, CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue());
		}
		if (solicitacaoAlcada.getDataReferencia() == null) {
			throw new BusinessException(internacionalizacaoService.getTranslate("SOLICITACAO_ALCADA.DATA_REFERENCIA", this.getIdioma())+msg, CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue());
		}
		if (solicitacaoAlcada.getTipoUtilizacao() == null) {
			throw new BusinessException(internacionalizacaoService.getTranslate("SOLICITACAO_ALCADA.TIPO_UTILIZACAO", this.getIdioma())+msg, CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue());
		}
		if (UtilString.isNullOrEmpty(solicitacaoAlcada.getRequisicaoOrigem())) {
			throw new BusinessException(internacionalizacaoService.getTranslate("SOLICITACAO_ALCADA.ORIGEM_SOLICITACAO", this.getIdioma())+msg, CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue());
		}   
		if (UtilString.isNullOrEmpty(solicitacaoAlcada.getIdentificador())) {
			throw new BusinessException(internacionalizacaoService.getTranslate("SOLICITACAO_ALCADA.ID_ORIGEM_SOLICITACAO", this.getIdioma())+msg, CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue());
		}          
	}

	protected void validaSolicitacao(SolicitacaoAlcada solicitacaoAlcada) {

		this.validaAtributos(solicitacaoAlcada);

		if (!UtilString.isNullOrEmpty(solicitacaoAlcada.getUsernameAprovador())) {
			Usuario aprovador = usuarioService.getUserByUsername(solicitacaoAlcada.getUsernameAprovador());
			if (!UtilObjeto.isReferencia(aprovador)) {
				throw new BusinessException(internacionalizacaoService.getTranslate("MSG.APROVADOR_NAO_ENCONTRADO", this.getIdioma()), CodigoErro.REGRA_NEGOCIO.getValue());
			}
			solicitacaoAlcada.setUsuarioAprovador(aprovador);
		}

		this.recuperaAlcada(solicitacaoAlcada);
		this.recuperaCentroResultado(solicitacaoAlcada);
		this.recuperaSolicitante(solicitacaoAlcada);
		this.recuperaTipoSolicitacao(solicitacaoAlcada);

		this.validaAtributos(solicitacaoAlcada);
	}

	protected void atribuiNivelAutoridade(final Alcada alcada, ResponsavelAlcada responsavelAlcada) {

		Usuario usuario = null;
		if (UtilObjeto.isReferencia(responsavelAlcada.getUsuario())) {
			usuario = responsavelAlcada.getUsuario();

			// find necessário para resolver problemas na serialização
			responsavelAlcada.setUsuario(usuarioService.find(usuario.getId()));

			if (responsavelAlcada.getDelegacao() != null && responsavelAlcada.getDelegacao()) {
				usuario = responsavelAlcada.getCentroResultadoDelegacao().getCentroResultadoResponsavelAlcada().getCentroResultadoResponsavel().getResponsavel().getPessoa().getUsuario();
			}
		}
		if (usuario != null) {
			for (GrupoUsuario grupoUsuario : usuario.getGrupoUsuarios()) {
				if (!grupoUsuario.isActive())
					continue;

				for (AlcadaNivelAutoridade alcadaNivelAutoridade : alcada.getNiveisAutoridades()) {
					if (!alcadaNivelAutoridade.isActive())
						continue;

					for (NivelAutoridadeGrupo nivelAutoridadeGrupo : alcadaNivelAutoridade.getNivelAutoridade().getGrupos()) {
						if (!nivelAutoridadeGrupo.isActive())
							continue;

						if (nivelAutoridadeGrupo.getGrupo().getId().equals(grupoUsuario.getGrupo().getId())) {
							if (!UtilObjeto.isReferencia(responsavelAlcada.getNivelAutoridade()) || responsavelAlcada.getNivelAutoridade().getHierarquia() > alcadaNivelAutoridade.getNivelAutoridade().getHierarquia()) {
								responsavelAlcada.setNivelAutoridade(alcadaNivelAutoridade.getNivelAutoridade());
							}
						}
					}
				}
			}
		}
	}

	protected void atribuiNivelAutoridade(final Alcada alcada, List<ResponsavelAlcada> responsaveis) {

		for (ResponsavelAlcada responsavelAlcada : responsaveis) {
			this.atribuiNivelAutoridade(alcada, responsavelAlcada);
		}
	}

	protected List<ResponsavelAlcada> getResponsaveisCentroResultado(final SolicitacaoAlcada solicitacaoAlcada) {

		List<ResponsavelAlcada> responsaveis = new ArrayList<ResponsavelAlcada>();

		for (CentroResultadoResponsavel centroResultadoResponsavel : solicitacaoAlcada.getCentroResultado().getResponsaveis()) {
			if (!centroResultadoResponsavel.isActive())
				continue;

			for (CentroResultadoResponsavelAlcada centroResultadoResponsavelAlcada : centroResultadoResponsavel.getAlcadas()) {
				if (!centroResultadoResponsavelAlcada.isActive())
					continue;

				if (centroResultadoResponsavelAlcada.getAlcada().getId().equals(solicitacaoAlcada.getAlcada().getId())) {
					if (!existeResponsavel(centroResultadoResponsavel.getResponsavel(), responsaveis)) {
						responsaveis.add(new ResponsavelAlcada(solicitacaoAlcada, centroResultadoResponsavel.getResponsavel(), null, null));
					}

					for (CentroResultadoDelegacao centroResultadoDelegacao : centroResultadoResponsavelAlcada.getDelegacoes()) {
						if (( centroResultadoDelegacao.getRevogada() == null || !centroResultadoDelegacao.getRevogada() ) && centroResultadoDelegacao.getDataInicioDelegacao().compareTo(solicitacaoAlcada.getDataReferencia()) <= 0 && centroResultadoDelegacao.getDataFimDelegacao().compareTo(solicitacaoAlcada.getDataReferencia()) >= 0 && !existeResponsavel(centroResultadoDelegacao.getDelegado(), responsaveis)) {

							boolean valido = true;
							if (centroResultadoDelegacao.getDominioTipoAbrangenciaDelegacao().getNome().equalsIgnoreCase("ESPECIFICAS")) {
								valido = false;
								for (CentroResultadoDelegacaoRequisicao centroResultadoDelegacaoRequisicao : centroResultadoDelegacao.getRequisicoes()) {
									if (centroResultadoDelegacaoRequisicao.getRequisicao().equals(solicitacaoAlcada.getRequisicaoOrigem())) {
										valido = true;
										break;
									}
								}
							}
							if (valido) {
								// TODO: Descobrir proque o centroResultadoDelegacao causa erro no JSON qdo chamado pelo Citsmart
								responsaveis.add(new ResponsavelAlcada(solicitacaoAlcada, centroResultadoDelegacao.getDelegado(), null, centroResultadoDelegacao));
							}
						}
					}
				}
			}
		}

		this.atribuiNivelAutoridade(solicitacaoAlcada.getAlcada(), responsaveis);
		return responsaveis;
	}

	protected boolean existeResponsavel(Colaborador colaborador, List<ResponsavelAlcada> responsaveis) {

		boolean result = false;
		for (ResponsavelAlcada responsavelAlcada : responsaveis) {
			if (responsavelAlcada.getColaborador().getId().equals(colaborador.getId())) {
				result = true;
				break;
			}
		}
		return result;
	}

	protected List<LimiteAprovacao> getLimitesAprovacao(SolicitacaoAlcada solicitacaoAlcada) {

		List<LimiteAprovacao> limitesAprovacao = new ArrayList<LimiteAprovacao>();

		// Obtem todas as alçadas associadas ao limite
		List<LimiteAprovacaoAlcada> limitesAprovacaoAlcada = limiteAprovacaoAlcadaService.findByAlcada(solicitacaoAlcada.getAlcada());

		if (limitesAprovacaoAlcada != null) {
			// Monta lista dos limites de cada alçada
			for (LimiteAprovacaoAlcada limiteAprovacaoAlcada : limitesAprovacaoAlcada) {
				if (!limitesAprovacao.contains(limiteAprovacaoAlcada.getLimiteAprovacao())) {
					limitesAprovacao.add(limiteAprovacaoAlcada.getLimiteAprovacao());
				}
			}
		}

		// Acumula valores por limite, alçada e nível de autoridade
		for (LimiteAprovacao limiteAprovacao : limitesAprovacao) {
			for (LimiteAprovacaoAlcada limiteAprovacaoAlcada : limiteAprovacao.getAlcadas()) {
				for (LimiteAprovacaoNivelAutoridade limiteAprovacaoNivelAutoridade : limiteAprovacao.getNiveisAutoridades()) {
					this.acumulaValores(solicitacaoAlcada, limiteAprovacaoAlcada.getAlcada(), limiteAprovacao, limiteAprovacaoNivelAutoridade.getNivelAutoridade());
				}
			}
		}

		return limitesAprovacao;
	}

	protected void acumulaValores(SolicitacaoAlcada solicitacaoAlcada, Alcada alcada, LimiteAprovacao limiteAprovacao, NivelAutoridade nivelAutoridade) {

		if (limiteAprovacao.getDominioTipoLimite().getCodigo().equals(FAIXA_VALOR)) {
			int ano = new Integer(UtilDate.getAno(solicitacaoAlcada.getDataReferencia().getTime()));
			int mes = new Integer(UtilDate.getMes(solicitacaoAlcada.getDataReferencia().getTime()));

			Collection<LimiteAprovacaoValor> valores = limiteAprovacao.getLimitesAprovacoesValores();
			if (valores != null) {
				for (LimiteAprovacaoValor limiteAprovacaoValor : valores) {
					if (!limiteAprovacaoValor.getDominioTipoUtilizacao().getCodigo().equals(solicitacaoAlcada.getTipoUtilizacao().getCodigo())) {
						continue;
					}

					float valorAcumulado = 0;

					if (!limiteAprovacaoValor.getDominioTipoLimiteValor().getCodigo().equals(INDIVIDUAL)) {
						valorAcumulado = solicitacaoAlcadaDao.getValorAcumulado(ano, limiteAprovacaoValor.getDominioTipoLimiteValor().getCodigo().equals(MENSAL) ? mes : null, alcada, solicitacaoAlcada.getCentroResultado(), solicitacaoAlcada.getTipoUtilizacao(), nivelAutoridade);
					}

					limiteAprovacaoValor.adicionaValorAcumulado(nivelAutoridade, valorAcumulado);
				}
			}
		}
	}

	protected void validaUsuario(SolicitacaoAlcada solicitacaoAlcada, List<LimiteAprovacao> limitesAprovacao, List<ResponsavelAlcada> responsaveis, HashMap<Long, Dominio> mapMotivosRejeicao) {

		for (ResponsavelAlcada responsavelAlcada : responsaveis) {
			if (!responsavelAlcada.getHabilitado())
				continue;

			if (!UtilObjeto.isReferencia(responsavelAlcada.getUsuario())) {
				responsavelAlcada.adicionaMotivoRejeicao(mapMotivosRejeicao.get(USUARIO_NAO_EXISTE));
			}
		}
	}

	protected void validaGrupos(SolicitacaoAlcada solicitacaoAlcada, List<LimiteAprovacao> limitesAprovacao, List<ResponsavelAlcada> responsaveis, HashMap<Long, Dominio> mapMotivosRejeicao) {

		for (ResponsavelAlcada responsavelAlcada : responsaveis) {
			if (!responsavelAlcada.getHabilitado())
				continue;

			if (!UtilObjeto.isReferencia(responsavelAlcada.getNivelAutoridade())) {
				responsavelAlcada.adicionaMotivoRejeicao(mapMotivosRejeicao.get(GRUPO_NAO_ASSOCIADO));
			}
		}
	}

	protected void validaLimiteValor(SolicitacaoAlcada solicitacaoAlcada, List<LimiteAprovacao> limitesAprovacao, List<ResponsavelAlcada> responsaveis, HashMap<Long, Dominio> mapMotivosRejeicao) {

		for (LimiteAprovacao limiteAprovacao : limitesAprovacao) {
			if (limiteAprovacao.getDominioTipoLimite().getCodigo().equals(FAIXA_VALOR)) {
				for (LimiteAprovacaoValor limiteAprovacaoValor : limiteAprovacao.getLimitesAprovacoesValores()) {
					if (solicitacaoAlcada.getTipoUtilizacao().getCodigo().equals(limiteAprovacaoValor.getDominioTipoUtilizacao().getCodigo())) {

						for (ValorAcumulado valorAcumulado : limiteAprovacaoValor.getValoresAcumulados()) {
							double valorRef = 0.0;
							if (limiteAprovacaoValor.getDominioTipoLimiteValor().getCodigo().equals(INDIVIDUAL)) {
								valorRef = solicitacaoAlcada.getValor().doubleValue();
							} else {
								valorRef = solicitacaoAlcada.getValor().doubleValue() + valorAcumulado.getValor();
							}

							valorAcumulado.setRejeitado(valorRef > limiteAprovacaoValor.getValorLimite().doubleValue());
							if (valorAcumulado.isRejeitado()) {
								for (ResponsavelAlcada responsavelAlcada : responsaveis) {
									if (!responsavelAlcada.getHabilitado())
										continue;

									if (responsavelAlcada.getNivelAutoridade().getId().equals(valorAcumulado.getNivelAutoridade().getId())) {
										responsavelAlcada.adicionaMotivoRejeicao(mapMotivosRejeicao.get(LIMITE_VALOR));
									}
								}
							}
						}
					}
				}

			}
		}
	}

	protected void validaSolicitante(SolicitacaoAlcada solicitacaoAlcada, List<LimiteAprovacao> limitesAprovacao, List<ResponsavelAlcada> responsaveis, HashMap<Long, Dominio> mapMotivosRejeicao) {

		if (!UtilObjeto.isReferencia(solicitacaoAlcada.getSolicitante()))
			return;

		for (AlcadaNivelAutoridade alcadaNivelAutoridade : solicitacaoAlcada.getAlcada().getNiveisAutoridades()) {
			if (!alcadaNivelAutoridade.isActive())
				continue;

			if (!alcadaNivelAutoridade.getPermiteAprovacaoPropria()) {
				for (ResponsavelAlcada responsavelAlcada : responsaveis) {
					if (!responsavelAlcada.getHabilitado())
						continue;

					if (responsavelAlcada.getNivelAutoridade().getId().equals(alcadaNivelAutoridade.getNivelAutoridade().getId())) {
						if (responsavelAlcada.getUsuario().getUsername().equalsIgnoreCase(solicitacaoAlcada.getSolicitante().getUsername())) {
							responsavelAlcada.adicionaMotivoRejeicao(mapMotivosRejeicao.get(PERMISSAO_AUTORIDADE));
						}
					}
				}
			}
		}
	}

	protected void validaAprovador(SolicitacaoAlcada solicitacaoAlcada, List<LimiteAprovacao> limitesAprovacao, List<ResponsavelAlcada> responsaveis, HashMap<Long, Dominio> mapMotivosRejeicao) {

		if (!UtilObjeto.isReferencia(solicitacaoAlcada.getAlcada().getGrupoResponsavel()))
			return;

		for (ResponsavelAlcada responsavelAlcada : responsaveis) {
			if (!responsavelAlcada.getHabilitado())
				continue;

			for (GrupoUsuario grupoUsuario : responsavelAlcada.getUsuario().getGrupoUsuarios()) {
				if (!grupoUsuario.isActive())
					continue;

				if (grupoUsuario.getGrupo().getId().equals(solicitacaoAlcada.getAlcada().getGrupoResponsavel().getId())) {
					responsavelAlcada.adicionaMotivoRejeicao(mapMotivosRejeicao.get(PERTENCE_GRUPO_RESPONSAVEL));
				}
			}
		}
	}

	protected void validaHierarquiaSolicitante(SolicitacaoAlcada solicitacaoAlcada, List<LimiteAprovacao> limitesAprovacao, List<ResponsavelAlcada> responsaveis, HashMap<Long, Dominio> mapMotivosRejeicao) {

		if (solicitacaoAlcada.getAlcada().getPermiteAprovacaoNivelInferior() == null || solicitacaoAlcada.getAlcada().getPermiteAprovacaoNivelInferior())
			return;

		if (!UtilObjeto.isReferencia(solicitacaoAlcada.getSolicitante()))
			return;

		int hierarquia = 0;
		for (ResponsavelAlcada responsavelAlcada : responsaveis) {
			if (!responsavelAlcada.getUsuario().getUsername().equalsIgnoreCase(solicitacaoAlcada.getSolicitante().getUsername()))
				continue;

			hierarquia = responsavelAlcada.getNivelAutoridade().getHierarquia();
			break;
		}

		if (hierarquia > 0) {
			for (ResponsavelAlcada responsavelAlcada : responsaveis) {
				if (!responsavelAlcada.getHabilitado())
					continue;

				if (responsavelAlcada.getNivelAutoridade().getHierarquia() > hierarquia) {
					responsavelAlcada.adicionaMotivoRejeicao(mapMotivosRejeicao.get(HIERARQUIA_SOLICITANTE));
				}
			}
		}
	}

	protected void validaNivelAutoridade(SolicitacaoAlcada solicitacaoAlcada, List<LimiteAprovacao> limitesAprovacao, List<ResponsavelAlcada> responsaveis, HashMap<Long, Dominio> mapMotivosRejeicao) {

		if (solicitacaoAlcada.getAlcada().getSomenteMenorHierarquia() == null || !solicitacaoAlcada.getAlcada().getSomenteMenorHierarquia())
			return;

		int hierarquia = 0;
		for (ResponsavelAlcada responsavelAlcada : responsaveis) {
			if (!responsavelAlcada.getHabilitado())
				continue;

			if (responsavelAlcada.getNivelAutoridade().getHierarquia() > hierarquia) {
				hierarquia = responsavelAlcada.getNivelAutoridade().getHierarquia();
			}
		}

		for (ResponsavelAlcada responsavelAlcada : responsaveis) {
			if (!responsavelAlcada.getHabilitado())
				continue;

			if (responsavelAlcada.getNivelAutoridade().getHierarquia() < hierarquia) {
				responsavelAlcada.adicionaMotivoRejeicao(mapMotivosRejeicao.get(HIERARQUIA_AUTORIDADE));
			}
		}
	}

	protected void validaResponsaveis(SolicitacaoAlcada solicitacaoAlcada, List<LimiteAprovacao> limitesAprovacao, List<ResponsavelAlcada> responsaveis) {

		HashMap<Long, Dominio> mapMotivosRejeicao = new HashMap<Long, Dominio>();
		Collection<Dominio> motivosRejeicao = dominioService.listarPorChave("motivoRejeicaoAlcada");
		for (Dominio dominio : motivosRejeicao) {
			mapMotivosRejeicao.put(dominio.getCodigo(), dominio);
		}

		this.validaGrupos(solicitacaoAlcada, limitesAprovacao, responsaveis, mapMotivosRejeicao);
		this.validaUsuario(solicitacaoAlcada, limitesAprovacao, responsaveis, mapMotivosRejeicao);
		this.validaLimiteValor(solicitacaoAlcada, limitesAprovacao, responsaveis, mapMotivosRejeicao);
		this.validaSolicitante(solicitacaoAlcada, limitesAprovacao, responsaveis, mapMotivosRejeicao);
		this.validaAprovador(solicitacaoAlcada, limitesAprovacao, responsaveis, mapMotivosRejeicao);
		this.validaHierarquiaSolicitante(solicitacaoAlcada, limitesAprovacao, responsaveis, mapMotivosRejeicao);
		this.validaNivelAutoridade(solicitacaoAlcada, limitesAprovacao, responsaveis, mapMotivosRejeicao);
	}

}
