package br.com.centralit.listener;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.stereotype.Component;

import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.InternacionalizacaoService;
import br.com.centralit.api.service.ModuloService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Internacionalizacao;
import br.com.centralit.framework.model.Modulo;
import br.com.centralit.framework.model.UtilStartup;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 25/02/2016 - 11:27:47
 * 
 * @version 1.0.0
 * 
 * @author rogerio.cassimiro
 * 
 */
@Component
public class StartupListenerAlcada extends UtilStartup implements ApplicationListener<ContextRefreshedEvent> {

	@Autowired
	private DominioService dominioService;

	@Autowired
	private InternacionalizacaoService internacionalizacaoService;

	@Autowired
	private ModuloService moduloService;

	@Override
	public void onApplicationEvent(final ContextRefreshedEvent event) {
		gerarInternacionalizacao();
	}

	/**
	 * 
	 * Método responsável por gerar internacionalização do módulo portal
	 * 
	 * @author rogerio.cassimiro
	 * 
	 */
	public void gerarInternacionalizacao() {

		dominioPT_BR = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_IDIOMA, Dominio.TIPO_IDIOMA_PT_BR_CODIGO);
		moduloSelecionado = this.moduloService.getModuloPorBaseUrl(BASE_URL_ALCADA, null);
		List<Internacionalizacao> internacionalizacaoList = new ArrayList<Internacionalizacao>();

		gerarMensagem(dominioPT_BR, moduloSelecionado, internacionalizacaoList);
		gerarValidacao(dominioPT_BR, moduloSelecionado, internacionalizacaoList);
		gerarLabel(dominioPT_BR, moduloSelecionado, internacionalizacaoList);

		internacionalizacaoService.saveListIfNotExist(internacionalizacaoList);
		Logger.getLogger(StartupListenerAlcada.class).info("Internacionalização executada - StartupListenerAlcada!");
	}

	private void gerarMensagem(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.MSG.CENTRO_RESULTADO_SEM_PARENT_REMOVE_RESPONSAVEIS", "Somente centros de resultado cujo atributo Centro de resultado superior estiver preenchido podem ter responsáveis!<br /> Favor remover o(s) responsável(is) ligado(s) a este centro de resultado para salvá-lo!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.MSG.NAO_FOI_ENCONTRADO_RESPONSAVEIS_PARA_ESTE_CENTRO_RESULTADO", "Não foi encontrado responsáveis para este centro de resultado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.MSG.SUBSTITUTO_NAO_PERMITIDO", "O substituto não pode ser um dos responsáveis pelo Centro de Resultado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.MSG.USUARIO_LOGADO_NAO_RESPONSAVEL_POR_NENHUM_CENTRO_RESULTADO", "Usuário logado não é responsável por nenhum centro de resultado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.MSG.CONFIRMA_REVOGACAO", "Confirma revogação?", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.MSG.ITEM_JA_REVOGADO", "Item já foi revogado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.MSG.ITEM_REGOVADO", "Item revogado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.MSG.SELECIONE_UM_ITEM_PARA_REVOGAR", "Selecione um item para revogar!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.MSG.CENTRO_CUSTO_RESPONSAVEL", "Para delegar a responsabilidade é necessário que o centro de resultado já tenha um responsável cadastrado!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.MSG.HINT_RESPONSAVEL_CENTRO_RESULTADO", "Somente centros de resultado cujo atributo Centro de resultado superior estiver preenchido podem ter responsáveis!", dominio, modulo));

	}

	private void gerarValidacao(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.VALIDACAO.ALERTA_OBRIGATORIOS_LIMITEAPROVACAO_VALOR", "É obrigatório informar o tipo de valor, a utilização e o valor!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.VALIDACAO.ALERTA_OBRIGATORIOS_NIVELAUTORIDADE_ALCADA", "É obrigatório informar o nivel de autoridade e a antecedência mínima para aprovação!", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.VALIDACAO.ALERTA_OBRIGATORIOS_RESPONSAVEIS", "É obrigatório informar o responsável e pelo menos uma alçada!", dominio, modulo));
	}

	private void gerarLabel(Dominio dominio, Modulo modulo, List<Internacionalizacao> internacionalizacaoList) {
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.ANTECEDENCIA_MINIMA_APROVACAO", "Antecedência mínima p/ aprovação (dias)", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.APROVACAO_NIVEL_INFERIOR", "Nível hierárquico inferior", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.CENTRO_RESULTADO_PAI", "Centro de resultado superior", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.DADOS_ALCADA", "Dados da alçada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.DADOS_CENTRO_RESULTADO", "Dados do centro de resultado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.DADOS_LIMITE_APROVACAO", "Dados do limite de aprovação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.DELEGADO_PARA", "Delegado para", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.DISPENSA_NOVA_APROVACAO", "% dispensa nova aprovação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.HISTORICO_RESPONSAVEIS", "Histórico de responsáveis", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.HISTORICO_SUBSTITUICOES", "Histórico de substituições", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.IDENTIFICADOR", "Identificador", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.LIMITES_VALOR", "Limites de valor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.PERIODO", "Período", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.PERMITE_APROVACAO_PROPRIA", "Permite aprovação própria", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.SUBSTITUIR", "Substituir", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.TIPO_LIMITE", "Tipo de limite", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.TIPO_UTILIZACAO", "Utilização", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.TIPO_VALOR", "Tipo de valor", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.ALCADA", "Alçada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.ALCADAS", "Alçadas", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.CENTRO_RESULTADO", "Centro de resultado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.DATA_REVOGACAO", " Data revogação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.DELEGACAO_ALCADA", "Delegação de alçada", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.DELEGAR", "Delegar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.HISTORICO_DELEGACAO", "Histórico de delegações", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.NIVEIS_AUTORIDADE", "Níveis de autoridade", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.REQUISICOES", "Requisições", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.REVOGAR", "Revogar", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.SUBSTITUTO", "Substituto", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.TIPO_ABRANGENCIA", "Abrangência do centro de resultado", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.ALCADA_CODIGO_SISTEMA_ORIGEM", "Código sistema origem", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.FLUXO_DELEGACAO", "Fluxo ESI para tratamento de delegação", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.GRUPO_ADMINISTRADOR", "Grupo administrador", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.PERMITE_APROVACAO_NIVEL_INFERIOR", "Aprovação em nível inferior quando solicitante igual ao autorizador", dominio, modulo));
		internacionalizacaoList.add(new Internacionalizacao("ALCADA.LABEL.SOMENTE_MENOR_HIERARQUIA", "Seleciona somente autoridade do nível mais baixo", dominio, modulo));


	}
}
