'use strict';

citApp.controller('AlcadaController', ['$scope', 'AlcadaRepository', 'NivelAutoridadeRepository', 'GrupoRepository', '$translate', '$filter', '$timeout', 'DominioRepository', 'FlowRepository',
                                       function AlcadaController($scope, AlcadaRepository, NivelAutoridadeRepository, GrupoRepository, 
        $translate, $filter, $timeout, DominioRepository, FlowRepository ) {

	$scope.fluxo = {};
	
    $scope.alcada = {
    	permiteAprovacaoNivelInferior : false
    };

    // Limpa formulário para novo cadastro
    $scope.resetForm = function() {
        $scope.limparAlcada();
        $scope.edit = true;
        $timeout(function(){
            $scope.alcadaForm.$submitted = false;
            $scope.alcadaForm.$setPristine();
        });
    };

    // Atualiza pagina de pesquisa
    $scope.atualizaPaginaPesquisa = function () {
        angular.element('#searchAlcada').scope().fetchResult();
    };

    // MODAL QUE CONFIRMA REMOVER DA ALCADA
    $scope.remove = function(alcada){
        $scope.alcada = alcada;
        $scope.$openModalConfirm({
            message: $translate.instant('MSG.CONFIRMA_EXCLUSAO'),
            callback: function () {
                AlcadaRepository.remove($scope.alcada).then(function() {

                    $scope.$modalConfirmInstance.dismiss('cancel');
                    $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
                    angular.element('#searchAlcada').scope().fetchResult();

                    $scope.resetForm();
                });
            }
        });
    };

    // SALVA A ALCADA
    $scope.saveOrUpdate = function(){
        $scope.alcadaForm.$submitted = true;

        //verifica se o formulario está valido para salvar
        if($scope.alcadaForm.$valid){

            $scope.setLoadingSalva(true);

            AlcadaRepository.save($scope.alcada).then(function(result) {
                $scope.showAlert("success", $translate.instant('MSG.REGISTRO_SALVO'));
                $scope.alcadaForm.$submitted = false;
                
                $scope.getAlcada(result.originalElement, true);
                
                $scope.setLoading(false);
            });
        }else{
            //Mensagem de erro de campos obrigatorios não preenchidos
            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
        }
    };

    // Limpa o formulario preenchido
    $scope.limparAlcada = function(){
        $scope.alcada = {        		
        		permiteAprovacaoNivelInferior : false,
        		fluxoDelegacao : ""
        };
    	$scope.fluxo = null;
    };

    // Consulta entidade e mostra no formulario
    $scope.getAlcada = function(alcada, edit){
        $scope.setLoadingGet(true);

        AlcadaRepository.get(alcada.id).then(function(result) {
            $scope.alcada = result.originalElement;
            
            $scope.alcada.niveisAutoridades = _.sortBy($scope.alcada.niveisAutoridades, function (alcada) {
            	return alcada.nivelAutoridade.nome;
            });
            
            if ($scope.alcada.fluxoDelegacao && $scope.alcada.fluxoDelegacao != '') {
	            FlowRepository.getByName($scope.alcada.fluxoDelegacao).then(function(result) {
	            	$scope.fluxo = result.originalElement.flow;
	            });
            }else{
            	$scope.fluxo = null;
            }
            $scope.edit = edit;
            $scope.setLoading(false);
        });
    };
    
    //FIND PARA AUTOCOMPLETE DO GRUPORESPONSAVEL
    $scope.findAutoCompleteGrupoResponsavel = function(value){
        return GrupoRepository.findAutoComplete('nome', value).then(function(result) {
            return result;
        });
    };
    
    //FIND PARA AUTOCOMPLETE DO GRUPOADMINISTRADOR
    $scope.findAutoCompleteGrupoAdministrador = function(value){
        return GrupoRepository.findAutoComplete('nome', value).then(function(result) {
            return result;
        });
    };
    
    //ALTERA TRUE PARA SIM E FALSE PARA NÃO
    $scope.aplicarFiltroBoolean = function(value){
    	return  $filter('booleanSimNao')(value);
    };
    
    //FIND PARA AUTOCOMPLETE DO NivelAutoridade
    $scope.findNivelAutoridade =  function(value){
      return NivelAutoridadeRepository.findAutoComplete('nome', value).then(function(result) {
    	  return $filter('idNotEqualObj')(result, $scope.alcada.niveisAutoridades, "nivelAutoridade");
      });
    };

    //#### INICIO METODOS ALCADANIVELAUTORIDADE ####//
    //ADICIONA O NIVELAUTORIDADE SELECIONADO
    $scope.adicionarAlcadaNivelAutoridade = function(){
      // SE NAO POSSUI ARRAY DE NIVELAUTORIDADE, CRIA O ATRIBUTO
      if($scope.alcadaNivelAutoridade != null && $scope.alcadaNivelAutoridade.nivelAutoridade != null &&
    		  $scope.alcadaNivelAutoridade.antecedenciaMinimaAprovacao != null){
	      if(!$scope.alcada.niveisAutoridades){
	          $scope.alcada.niveisAutoridades = [];
	      }
	        
	      $scope.alcada.niveisAutoridades.push({nivelAutoridade : $scope.alcadaNivelAutoridade.nivelAutoridade,
	                                            permiteAprovacaoPropria : $scope.alcadaNivelAutoridade.permiteAprovacaoPropria,
	                                            antecedenciaMinimaAprovacao : $scope.alcadaNivelAutoridade.antecedenciaMinimaAprovacao});
	      $timeout(function(){
	        $scope.alcadaNivelAutoridade.nivelAutoridade = null;
	        $scope.alcadaNivelAutoridade.permiteAprovacaoPropria = false;
	        $scope.alcadaNivelAutoridade.antecedenciaMinimaAprovacao = '';
	      });
       }
       else{
          //Mensagem de erro de campos obrigatorios não preenchidos
          $scope.showAlert('error', $translate.instant('ALCADA.VALIDACAO.ALERTA_OBRIGATORIOS_NIVELAUTORIDADE_ALCADA'), " ", false);
       }
    };
    
    //REMOVE O NIVELAUTORIDADE SELECIONADO
    $scope.removerAlcadaNivelAutoridade = function(){
      if(!$scope.nivelAutoridadeChecked) {
        $scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
        return;
      } else {
        $scope.$openModalConfirm({
          message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'),
          callback: function () {
            $scope.$modalConfirmInstance.dismiss('cancel');
            $scope.alcada.niveisAutoridades.splice($scope.nivelAutoridadeChecked.$index, 1);
            $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
          }
        });
      }
    };
    
	$scope.findFlow = function(value) {
		var flowVH = {
			nome : value
		};		
		return FlowRepository.findFlow(flowVH).then(function(result) {
			var flows = [];
			if (result && result.length > 0) {
				for (var i = 0; i < result.length; i++) {
					var flow = result[i].originalElement;
					if (flow.flowApplication == 'SERVICE_INTEGRATION') {
						flows.push(result[i]);
					}
				}
			}
			return flows;
		});
	};

	$scope.limparAutoCompleteFluxo = function() {
		$scope.alcada.fluxoDelegacao = null;
	};

	$scope.setFluxo = function (item) {
		if(item && item.id) {
			$scope.fluxo = item;
			$scope.alcada.fluxoDelegacao = item.name;
		} 
	};

    
    //#### FIM METODOS ALCADANIVELAUTORIDADE ####//
}]);