'use strict';

citApp.controller('CentroResultadoController', ['$scope', 'CentroResultadoRepository', 'CentroResultadoResponsavelRepository', 'PessoaRepository', 'AlcadaRepository', '$translate', '$timeout', '$filter',
		function CentroResultadoController($scope, CentroResultadoRepository, CentroResultadoResponsavelRepository, PessoaRepository, AlcadaRepository, $translate, $timeout, $filter) {

    $scope.centroResultado = {};
    $scope.historicoResponsaveis = [];
    $scope.alcadas = [];
    // Limpa formulário para novo cadastro
    $scope.resetForm = function() {
        $scope.limparCentroResultado();
		$scope.findAlcadaSource();
        $scope.edit = true;
        $timeout(function(){
            $scope.centroResultadoForm.$submitted = false;
            $scope.centroResultadoForm.$setPristine();
        });
    };

    // Atualiza pagina de pesquisa
    $scope.atualizaPaginaPesquisa = function () {
        angular.element('#searchCentroResultado').scope().fetchResult();
    };

    // MODAL QUE CONFIRMA REMOVER Do CENTRO DE RESULTADO
    $scope.remove = function(centroResultado){
        $scope.centroResultado = centroResultado;
        $scope.$openModalConfirm({
            message: $translate.instant('MSG.CONFIRMA_EXCLUSAO'),
            callback: function () {
                CentroResultadoRepository.remove($scope.centroResultado).then(function() {

                    $scope.$modalConfirmInstance.dismiss('cancel');
                    $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
                    angular.element('#searchCentroResultado').scope().fetchResult();

                    $scope.resetForm();
                });
            }
        });
    };

    // SALVA O CentroResultado
    $scope.saveOrUpdate = function(){
        $scope.centroResultadoForm.$submitted = true;

       // $scope.verificaResponsaveis();
        //verifica se o formulario está valido para salvar
        if($scope.centroResultadoForm.$valid){

            if (!$scope.centroResultado.centroResultadoParent && $scope.centroResultado.responsaveis && $scope.centroResultado.responsaveis.lenght){

            	$scope.showAlert('warning', $translate.instant('ALCADA.MSG.CENTRO_RESULTADO_SEM_PARENT_REMOVE_RESPONSAVEIS'), " ", false);

            } else {
            	$scope.setLoadingSalva(true);

                CentroResultadoRepository.save($scope.centroResultado).then(function(result) {
                    $scope.centroResultado = result.originalElement;
                    $scope.showAlert("success", $translate.instant('MSG.REGISTRO_SALVO'));
                    $scope.centroResultadoForm.$submitted = false;
                    $scope.getCentroResultado(result.originalElement, true);

                    $scope.setLoading(false);
                });
            }
        }else{
            //Mensagem de erro de campos obrigatorios não preenchidos
            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
        }
    };

    // Limpa o formulario preenchido
    $scope.limparCentroResultado = function(){
        $scope.alcadas = [];
        $scope.centroResultado = {};
        $scope.historicoResponsaveis = [];
        $scope.responsavel = null;
        $scope.editResponsavel = false;
    };

    // Consulta entidade e mostra no formulario
    $scope.getCentroResultado = function(centroResultado, edit){
        $scope.setLoadingGet(true);
        $scope.limparCentroResultado();

        CentroResultadoRepository.get(centroResultado.id).then(function(result) {
            $scope.centroResultado = result.originalElement;
            $scope.buscaHistoricoResponsaveis(centroResultado);
    		$scope.findAlcadaSource();
            $scope.edit = edit;
            $scope.setLoading(false);
        });
    };

    $scope.buscaHistoricoResponsaveis = function(centroResultado){
		CentroResultadoResponsavelRepository.buscaHistoricoResponsaveis(centroResultado.id).then(function(result) {
	        $scope.historicoResponsaveis = result;
	    });
	};

    $scope.findAutoCompleteCentroResultado = function(value){
        return CentroResultadoRepository.findAutoComplete('nome', value).then(function(result) {

        	var resultFiltered = $filter('idNotEqual')(result, [$scope.centroResultado]);

            return resultFiltered;
        });
    };

    //#### INICIO METODOS RESPONSAVEL ALCADA CENTRORESULTADO
    $scope.findAutoCompleteAlcada = function(value){
      return AlcadaRepository.findAutoComplete('nome', value).then(function(result) {
          return result;
      });
    };

    $scope.findAutoCompleteResponsavel = function(value) {
      return PessoaRepository.findColaboradorPorNome(value).then(function(result) {

    	  var responsaveis = [];

    	  if($scope.centroResultado.responsaveis) {
    		  $scope.centroResultado.responsaveis.forEach(function (item) {
        		  responsaveis.push(item.responsavel);
        	  });
    	  }

    	  var resultFiltered = $filter('idNotEqual')(result, responsaveis);

          return resultFiltered;
      });
    };

    //ADICIONA O RESPONSAVEL SELECIONADO
    $scope.adicionarResponsavel = function(){
      // SE NAO POSSUI ARRAY DE RESPONSAVEIS, CRIA O ATRIBUTO
      if($scope.responsavel != null && $scope.alcadas.length > 0){

         if(!$scope.centroResultado.responsaveis){
             $scope.centroResultado.responsaveis = [];
         }

         var indexResponsavel = _.findIndex($scope.centroResultado.responsaveis, {responsavel: {id: $scope.responsavel.id}});
         if(indexResponsavel >= 0) {
        	 $scope.centroResultado.responsaveis[indexResponsavel].responsavel = $scope.responsavel;

        	// Seta data inativo nos itens removidos pelo usuario
        	 $scope.centroResultado.responsaveis[indexResponsavel].alcadas.forEach(function (item) {
        		 var indexAlcada = _.findIndex($scope.alcadas, {alcada: {id: item.alcada.id}});

        		 if(indexAlcada < 0 && item.id) {
        			 item.dataInativo = $filter('date')(new Date(), 'dd/MM/yyyy');
        		 } else if(item.id) {
        			 item.dataInativo = null;
        		 }
        	 });

        	 // Adiciona os itens selecionados que ainda não foi salvo
        	 $scope.alcadas.forEach(function (item) {
        	 	if(!item.id) {
        	 		$scope.centroResultado.responsaveis[indexResponsavel].alcadas.push(item);
        	 	}
        	 });
         } else {
        	 $scope.centroResultado.responsaveis.push({alcadas : $scope.alcadas,
                 responsavel : $scope.responsavel, dataInativo: null});
         }

         $timeout(function(){
           $scope.alcadas = [];
           $scope.findAlcadaSource();
           $scope.responsavel = null;
           $scope.editResponsavel = false;
         });
      }else{
          //Mensagem de erro de campos obrigatorios não preenchidos
          $scope.showAlert('error', $translate.instant('ALCADA.VALIDACAO.ALERTA_OBRIGATORIOS_RESPONSAVEIS'), " ", false);
      }
    };

    //REMOVE O RESPONSAVEL SELECIONADO
    $scope.removerResponsavel = function(){
      if(!$scope.responsavelChecked) {
        $scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
        return;
      } else {
    	  $scope.$openModalConfirm({
              message: $translate.instant('MSG.CONFIRMA_EXCLUSAO'),
              callback: function () {
            	  if($scope.responsavelChecked.id != undefined && $scope.responsavelChecked.id){
                	 CentroResultadoResponsavelRepository.removeById($scope.responsavelChecked.id).then(function() {
	                	  $scope.$modalConfirmInstance.dismiss('cancel');
	                      $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
	                      $scope.centroResultado.responsaveis.splice($scope.responsavelChecked.$index, 1);
                     });
            	  } else {
            		  $scope.$modalConfirmInstance.dismiss('cancel');
                      $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
                      $scope.centroResultado.responsaveis.splice($scope.responsavelChecked.$index, 1);
            	  }
              }
    	  });
      }
    };

    // EDITAR O RESPONSAVEL SELECIONADO
    $scope.editarResponsavel = function () {
    	if(!$scope.responsavelChecked) {
    		$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_EDITADO'));
            return;
    	} else {
    		$scope.responsavel = $scope.responsavelChecked.responsavel;
    		$scope.alcadas = _.where($scope.responsavelChecked.alcadas, {dataInativo: null});
    		$scope.editResponsavel = true;

    		removeAttributesGrupoAlcada();

    		$scope.responsavelChecked.alcadas.forEach(function (item) {
    			var indexAlcada = _.findIndex($scope.grupoAlcadaSource, {alcada: {id: item.alcada.id}});
    			$scope.grupoAlcadaSource[indexAlcada] = item;
    		});
    	}
    };

    var removeAttributesGrupoAlcada = function () {
    	var grupoAlcadaCopy = angular.copy($scope.grupoAlcadaSource);
    	$scope.grupoAlcadaSource = [];

    	grupoAlcadaCopy.forEach(function (item) {
    		$scope.grupoAlcadaSource.push( _.pick(item, 'alcada') );
    	});
    };

    //#### FIM METODOS RESPONSAVEL ALCADA CENTRORESULTADO

	$scope.grupoAlcadaSource = [];

	$scope.filterCriteriaAlcada = {
		start : 1,
		dir : 'asc',
		sort : 'id',
		limit : 10,
		fields: ['id', 'nome']
	};

	//Método responsável por listar as alcadas
	$scope.findAlcadaSource = function(){
		$scope.grupoAlcadaSource = [];
		AlcadaRepository.getListPage($scope.filterCriteriaAlcada).then(function(result) {
			//Percorre a lista de alcada para montar a lista de grupoAlcada
			result.originalElement.objects.forEach(function (item, index) {
				var grupoAlcada = {};
				grupoAlcada.alcada = item;
				$scope.grupoAlcadaSource.push(grupoAlcada);
			});
		});
	};
}]);