'use strict';

citApp.controller('CentroResultadoDelegacaoController', ['$scope', 'CentroResultadoDelegacaoRepository', 'PessoaRepository', 'CentroResultadoRepository', 'DominioRepository', 'AlcadaRepository', 'CentroResultadoResponsavelRepository', 'CentroResultadoResponsavelAlcadaRepository', '$translate', '$timeout', '$filter',
		function CentroResultadoDelegacaoController($scope, CentroResultadoDelegacaoRepository, PessoaRepository, CentroResultadoRepository, DominioRepository, AlcadaRepository, CentroResultadoResponsavelRepository, CentroResultadoResponsavelAlcadaRepository, $translate, $timeout, $filter) {
    $scope.centroResultadoDelegacao = {};
    $scope.alcadasSelecionadas = [];
    $scope.idsAlcadasSelecionados = [];
    $scope.historicoDelegacoes = [];
    $scope.colaboradoresSelecionados = [];
    
    $scope.TIPO_ABRANGENCIA_DELEGACAO_NOVAS_E_ANDAMENTO_CODIGO = 1;
    $scope.TIPO_ABRANGENCIA_DELEGACAO_NOVAS = 2;
    $scope.TIPO_ABRANGENCIA_DELEGACAO_ESPECIFICAS = 3;
    
    // Limpa formulário para novo cadastro
    $scope.resetForm = function() {
        $scope.alcadas = [];
        $scope.filter.centroResultado = null;
        $scope.idsAlcadasSelecionados = [];
        $scope.historicoDelegacoes = [];
        $scope.alcadasSelecionadas = [];
        $scope.limparCentroResultadoDelegacao();
        $scope.edit = true;
        $timeout(function(){
            $scope.centroResultadoDelegacaoForm.$submitted = false;
            $scope.centroResultadoDelegacaoForm.$setPristine();
        });
    };
    
    // Limpa o formulario preenchido
    $scope.limpar = function(){
        $scope.centroResultadoDelegacao = {};
        $scope.resetForm();
    };
    
    // Atualiza pagina de pesquisa
    $scope.atualizaPaginaPesquisa = function () {
        angular.element('#searchCentroResultadoDelegacao').scope().fetchResult();
    };

    // MODAL QUE CONFIRMA REMOVER DA CIDADE
    $scope.remove = function(centroResultadoDelegacao){
        $scope.centroResultadoDelegacao = centroResultadoDelegacao;
        $scope.$openModalConfirm({
            message: $translate.instant('MSG.CONFIRMA_EXCLUSAO'),
            callback: function () {
                CentroResultadoDelegacaoRepository.remove($scope.centroResultadoDelegacao).then(function() {

                    $scope.$modalConfirmInstance.dismiss('cancel');
                    $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
                    angular.element('#searchCentroResultadoDelegacao').scope().fetchResult();

                    $scope.resetForm();
                });
            }
        });
    }; 

    // Consulta entidade e mostra no formulario
    $scope.getCentroResultadoDelegacao = function(centroResultadoDelegacao, edit){
        $scope.setLoadingGet(true);

        CentroResultadoDelegacaoRepository.get(centroResultadoDelegacao.id).then(function(result) {
            $scope.centroResultadoDelegacao = result.originalElement;
            $scope.buscaHistoricoDelegacoes($scope.centroResultadoDelegacao.centroResultadoResponsavelAlcada.centroResultadoResponsavel.centroResultado.id);
            $scope.edit = edit;
            $scope.setLoading(false);
        });
    };

    //FIND AUTOCOMPLETE ALCADA
    $scope.findAutoCompleteAlcada = function(value){
        return AlcadaRepository.findAutoComplete('nome', value).then(function(result) {
            return result;
        });
    };
    
    //FIND AUTOCOMPLETE CENTRORESULTADO
    $scope.findAutoCompleteCentroResultado = function(value){
        return CentroResultadoRepository.findAutoComplete('nome', value).then(function(result) {
            return result;
        });
    };
    
    //FIND AUTOCOMPELTE RESPONSAVEL
    $scope.findAutoCompleteColaborador = function(value){
        return PessoaRepository.findColaboradorPorNome(value).then(function(result) {
        	// Filtra para nao mostrar os colaboradores que foram selecionados
            return $filter('idNotEqual')(result, $scope.colaboradoresSelecionados);
        });
    };
    
    //BUSCA OS DOMINIOS tipoAbrangenciaDelegacao
    DominioRepository.findAllDominio('tipoAbrangenciaDelegacao').then(function(result) {
        $scope.dominioTipoAbrangenciaDelegacaoList = result;
    }); 
    
 
    //#### INICIO METODOS TABELA ALCADAS PAGINADA ####//
    // DEFINE CABEÇALHOS DA LISTAGEM
    $scope.headers = [ {title : $translate.instant('LABEL.RESPONSAVEL'), value : 'centroResultadoResponsavel.responsavel.pessoa.nome' } , 
                         {title : $translate.instant('ALCADA.LABEL.ALCADA'), value : 'alcada.nome' } ];

    $scope.totalPages = 10;
    $scope.totalItens = 0;
    $scope.limit = 2;
    
    //FILTRO CRITERIA 
    $scope.filterCriteria = {
            start : 1,
            dir : 'desc',
            sort : 'id',
            limit : 10,
            fields: ['id', 'centroResultadoResponsavel.responsavel.pessoa.nome', 'centroResultadoResponsavel.responsavel.pessoa.id', 'alcada.nome' ],
            filters : [ {type : 'numeric', field : 'centroResultadoResponsavel.centroResultado.id' }]
    };
    
    $scope.fetchResult = function(page) {
        $scope.setLoadingPesquisa(true);
        $scope.itemSelecionado = undefined;
        

        $scope.alcadas = [];
        
        $scope.idsAlcadasSelecionados = [];
        $scope.historicoDelegacoes = [];
        $scope.alcadasSelecionadas = [];
        $scope.limparCentroResultadoDelegacao();
   
        $timeout(function(){
            $scope.centroResultadoDelegacaoForm.$submitted = false;
            $scope.centroResultadoDelegacaoForm.$setPristine();
        });        
        
        if($scope.filter.centroResultado && $scope.filter.centroResultado.id){
          $scope.filterCriteria.filters[0].value = $scope.filter.centroResultado.id;
          $scope.buscaHistoricoDelegacoes($scope.filter.centroResultado.id);
        }else{
          $scope.filterCriteria.filters[0].value = '';
        }

        CentroResultadoResponsavelAlcadaRepository.getListPage($scope.filterCriteria).then(function(result) {
          $scope.alcadas = result.originalElement.objects;
          $scope.totalPages = result.originalElement.totalPages;
          $scope.totalItens =  result.originalElement.totalItens;
          $scope.scrollToCadastro('#tableAlcadas');
          $scope.selecionarAlcadas = false;
          $scope.setLoading(false);
          
          if(!result.originalElement.objects || (result.originalElement.objects && result.originalElement.objects.length <= 0)) {
        	  $scope.showAlert('warning', $translate.instant('ALCADA.MSG.NAO_FOI_ENCONTRADO_RESPONSAVEIS_PARA_ESTE_CENTRO_RESULTADO'));
          }
        }, function() {
          $scope.alcadas = [];
          $scope.totalPages = 0;
          $scope.totalItens = 0;
          $scope.setLoading(false);
        });
    };
    //#### FIM METODOS TABELA ALCADAS PAGINADA ####//
    
    //#### INICIO METODOS DELEGAR ####//
    //SELECIONA UMA ALCADA - RESPONSAVEL
    $scope.selecionarAlcada = function(alcada, $index){
    	$timeout(function(){
    		if (!alcada.$checked) {
    			var indexIdAlcadaSelecionada = _.indexOf($scope.idsAlcadasSelecionados, alcada.id);
    			if(indexIdAlcadaSelecionada >= 0) {
    				$scope.idsAlcadasSelecionados.splice(indexIdAlcadaSelecionada, 1);
    			}
    			
    			var indexAlcadaSelecionada = _.findIndex($scope.alcadasSelecionadas, {id: alcada.id});
    			if(indexAlcadaSelecionada >= 0) {
    				$scope.alcadasSelecionadas.splice(indexAlcadaSelecionada, 1);
    			}
    		}else {
    			$scope.idsAlcadasSelecionados.push(alcada.id);
    			$scope.alcadasSelecionadas.push(alcada);
    		}
    	});
    };
    
    DominioRepository.findAllDominio('tipoAbrangenciaCentroResultadoDelegacao').then(function(result) {
    	$scope.dominioTipoAbrangenciaList = result;
    });
    
    $scope.iniciaModal = function(){
    	$scope.limparCentroResultadoDelegacao();
    	getColaboradoresSelecionados();
		$scope.$openModal('/cit-alcada-web/html/centroResultadoDelegacao/dialogCentroResultadoDelegacao.html', 'lg');
    };

    var getColaboradoresSelecionados = function() {
    	$scope.colaboradoresSelecionados = [];
    	
    	var colaborador = 0;
    	
    	for(var i = 0; i < $scope.alcadasSelecionadas.length; i++) {
    		colaborador = {id: $scope.alcadasSelecionadas[i]['centroResultadoResponsavel.responsavel.pessoa.id']};
    		
    		if(!_.findWhere($scope.colaboradoresSelecionados, colaborador)) {
    			$scope.colaboradoresSelecionados.push(colaborador);
    		}
    	}
    };
    
    //DELEGA UMA ALCADA - RESPONSAVEL
    $scope.delegarAlcada = function(formDelegacaoAlcada) {    	
    	formDelegacaoAlcada.$submitted = true;

		if(formDelegacaoAlcada.$valid){
            var dataInicio = $scope.centroResultadoDelegacao.dataInicioDelegacao;
            var dataFim = $scope.centroResultadoDelegacao.dataFimDelegacao;
            
            if(isPeriodoValido(dataInicio, dataFim)){
            	var centroResultadoDelegacaoList = [];
    			for (var i = 0; i < $scope.alcadasSelecionadas.length; i++) {
    				centroResultadoDelegacaoList.push(montarObjeto($scope.alcadasSelecionadas[i].id));
    			}
    			
    			$scope.saveList(centroResultadoDelegacaoList);
    			
    		} else {
              $scope.showAlert("error", $translate.instant("VALIDACAO.PERIODO_INVALIDO_DATAINICIO_MAIOR_DATAFIM") + " ", false );
            }
		}else{
			$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'));
		}
	};
	
	$scope.saveList = function (centroResultadoDelegacaoList) {
		$scope.setLoadingSalva(true);
		
		CentroResultadoDelegacaoRepository.saveList(centroResultadoDelegacaoList).then(function (result) {
			$scope.showAlert("success", $translate.instant('MSG.REGISTRO_SALVO'));
			$scope.centroResultadoDelegacaoForm.$submitted = false;
			
			$scope.buscaHistoricoDelegacoes($scope.filter.centroResultado.id);
			$scope.setLoading(false);
			
			if($scope.$modalInstance) {
				$scope.$modalInstance.dismiss('cancel');
			}
		});
	};
	
	$scope.limparCentroResultadoDelegacao = function(){
		$scope.centroResultadoDelegacao = {};
	};
	//MONTA O OBJETO DELEGACAO PARA SALVAR
    function montarObjeto(idCentroResultadoResponsavelAlcada){   		
    	var centroResultadoDelegacao = angular.copy($scope.centroResultadoDelegacao);
    	
    	centroResultadoDelegacao.centroResultadoResponsavelAlcada = {
    		id: idCentroResultadoResponsavelAlcada
    	};
    	
    	montaObjetoRequisicoes(centroResultadoDelegacao);
    	
    	return centroResultadoDelegacao;
    }
    
    var montaObjetoRequisicoes = function(centroResultadoDelegacao) {
    	if(centroResultadoDelegacao.requisicoes && centroResultadoDelegacao.requisicoes.length) {
    		var requisicoes = [];
    		for(var i = 0; i < centroResultadoDelegacao.requisicoes.length; i++) {
    			var requisicao = {
    				requisicao: centroResultadoDelegacao.requisicoes[i]
    			};
    			
    			requisicoes.push(requisicao);
    		}
    		
    		centroResultadoDelegacao.requisicoes = requisicoes;
    	}
    };
    
    //#### FIM METODOS DELEGACAO####//
    
    //BUSCA TODAS AS DELEGACOES DO CENTRO DE RESULTADO
    $scope.buscaHistoricoDelegacoes = function(centroResultadoId){    	
		CentroResultadoDelegacaoRepository.buscaHistoricoDelegacoes(centroResultadoId).then(function(result) {
		    $scope.historicoDelegacoes  = result;            
	    });
	};
	
    $scope.revogarDelegacao = function(){
        if(!$scope.delegadoChecked) {
          $scope.showAlert('warning', $translate.instant('ALCADA.MSG.SELECIONE_UM_ITEM_PARA_REVOGAR'));
          return;
        } 
        if($scope.delegadoChecked.dataRevogacao) {
            $scope.showAlert('warning', $translate.instant('ALCADA.MSG.ITEM_JA_REVOGADO'));
            return; 
    	} else {
      	  $scope.$openModalConfirm({
                message: $translate.instant('ALCADA.MSG.CONFIRMA_REVOGACAO'),
                callback: function () {
                    CentroResultadoDelegacaoRepository.revogar($scope.delegadoChecked.id).then(function (result) {
                    	$scope.$modalConfirmInstance.dismiss('cancel');
                    	$scope.showAlert("success", $translate.instant('ALCADA.MSG.ITEM_REGOVADO'));
                    	
                    	$scope.delegadoChecked.dataRevogacao = dataHoje();
                    });
                }  
      	  });
        }
      };
    function dataHoje() {
    	return $filter('date')(new Date(), 'dd/MM/yyyy');
    };
}]);
