'use strict';

citApp.controller('LimiteAprovacaoController', ['$scope', 'LimiteAprovacaoRepository', 'AlcadaRepository', 'NivelAutoridadeRepository', '$filter', '$translate', '$timeout', 'DominioRepository',
		function LimiteAprovacaoController($scope, LimiteAprovacaoRepository, AlcadaRepository, NivelAutoridadeRepository, $filter, $translate, $timeout, DominioRepository ) {

	$scope.TIPO_LIMITE_FAIXA_VALORES_CODIGO = 1;
	$scope.TIPO_LIMITE_QUALQUER_VALOR_CODIGO = 2;
	$scope.TIPO_LIMITE_NAO_APLICA_CODIGO = 3;
	
    $scope.limiteAprovacao = {};
    
    $scope.alcadasTarget = [];

    DominioRepository.findAllDominio('tipoAbrangenciaCentroResultado').then(function(result) {
      $scope.dominioTipoAbrangenciaList = result;
    }); 
    DominioRepository.findAllDominio('tipoLimite').then(function(result) {
      $scope.dominioTipoLimiteList = result;
    }); 
  
    // Limpa formulário para novo cadastro
    $scope.resetForm = function() {
        
		$scope.limiteAprovacao = {				
				alcadas : [],		
				niveisAutoridades : []
		};
		
    	$scope.limparLimiteAprovacao();
		$scope.findAlcadaSource();		
		$scope.findNivelAutoridadeSource();
		
        $scope.edit = true;
        $timeout(function(){
            $scope.limiteAprovacaoForm.$submitted = false;
            $scope.limiteAprovacaoForm.$setPristine();
        });
    };

    // Atualiza pagina de pesquisa
    $scope.atualizaPaginaPesquisa = function () {
        angular.element('#searchLimiteAprovacao').scope().fetchResult();
    };

    // MODAL QUE CONFIRMA REMOVER DA LIMITEAPROVACAO
    $scope.remove = function(limiteAprovacao){
        $scope.limiteAprovacao = limiteAprovacao;
        $scope.$openModalConfirm({
            message: $translate.instant('MSG.CONFIRMA_EXCLUSAO'),
            callback: function () {
                LimiteAprovacaoRepository.remove($scope.limiteAprovacao).then(function() {

                    $scope.$modalConfirmInstance.dismiss('cancel');
                    $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
                    angular.element('#searchLimiteAprovacao').scope().fetchResult();

                    $scope.resetForm();
                });
            }
        });
    };

    // SALVA O LIMITEAPROVACAO
    $scope.saveOrUpdate = function(){
        $scope.limiteAprovacaoForm.$submitted = true;

        //verifica se o formulario esta valido para salvar
        if($scope.limiteAprovacaoForm.$valid){

            $scope.setLoadingSalva(true);

            LimiteAprovacaoRepository.save($scope.limiteAprovacao).then(function(result) {
                $scope.showAlert("success", $translate.instant('MSG.REGISTRO_SALVO'));
                $scope.limiteAprovacaoForm.$submitted = false;
                
                $scope.getLimiteAprovacao(result.originalElement, true);
                
                $scope.setLoading(false);
            });
        }else{
            //Mensagem de erro de campos obrigatorios nao preenchidos
            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
        }
    };

    // Limpa o formulario preenchido
    $scope.limparLimiteAprovacao = function(){
        $scope.limiteAprovacao = {limitesAprovacoesValores: [], niveisAutoridades: [], alcadas: [] };
    };

    // Consulta entidade e mostra no formulario
    $scope.getLimiteAprovacao = function(limiteAprovacao, edit){
        $scope.setLoadingGet(true);
        $scope.limparLimiteAprovacao();

        LimiteAprovacaoRepository.get(limiteAprovacao.id).then(function(result) {
            $scope.limiteAprovacao = result.originalElement;
            
            $scope.limiteAprovacao.limitesAprovacoesValores = _.sortBy($scope.limiteAprovacao.limitesAprovacoesValores, function (limiteAprovacao) {
            	return limiteAprovacao.dominioTipoLimiteValor.descricao;
            });
            
            $scope.edit = edit;
    		$scope.findAlcadaSource();		
    		$scope.findNivelAutoridadeSource();
            $scope.setLoading(false);
        });
    };

    //#### INICIO METODOS DO LIMITEAPROVAOVALOR ####//
    // Busca dominios tipoUtilizacao
    DominioRepository.findAllDominio('tipoUtilizacao').then(function(result) {
      $scope.dominioTipoUtilizacaoList = result;
    }); 
    // Busca dominios tipoLimiteValor
    DominioRepository.findAllDominio('tipoLimiteValor').then(function(result) {
      $scope.dominioTipoLimiteValorList = result;
    }); 
    
    //ADICIONA O LIMITEAPROVAOVALOR SELECIONADO
    $scope.adicionarLimiteAprovacaoValor = function(){
       if($scope.limiteAprovacaoValor != null && $scope.limiteAprovacaoValor.dominioTipoUtilizacao != null &&
    		   $scope.limiteAprovacaoValor.dominioTipoLimiteValor != null && $scope.limiteAprovacaoValor.valor != null){
    	 // SE NAO POSSUI ARRAY DE LIMITEAPROVACAOVALORES, CRIA O ATRIBUTO
	      if(!$scope.limiteAprovacao.limitesAprovacoesValores){
	          $scope.limiteAprovacao.limitesAprovacoesValores = [];
	      }
	      
	      $scope.limiteAprovacao.limitesAprovacoesValores.push({dominioTipoUtilizacao : $scope.limiteAprovacaoValor.dominioTipoUtilizacao,
                  dominioTipoLimiteValor : $scope.limiteAprovacaoValor.dominioTipoLimiteValor,
                  valorLimite : $scope.limiteAprovacaoValor.valor, dataInativo: null});
	      
	      $timeout(function(){
	        $scope.limiteAprovacaoValor.dominioTipoUtilizacao = null;
	        $scope.limiteAprovacaoValor.dominioTipoLimiteValor = null;
	        $scope.limiteAprovacaoValor.valor = null;
	      });

      }else{
         //Mensagem de erro de campos obrigatorios não preenchidos
         $scope.showAlert('error', $translate.instant('ALCADA.VALIDACAO.ALERTA_OBRIGATORIOS_LIMITEAPROVACAO_VALOR'), " ", false);
      }
    };
    
    //REMOVE O LIMITEAPROVAOVALOR SELECIONADO
    $scope.removerLimiteAprovacaoValor = function(){
      if(!$scope.limiteAprovacaoValorChecked) {
        $scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
        return;
      } else {
        $scope.$openModalConfirm({
          message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'),
          callback: function () {
            $scope.$modalConfirmInstance.dismiss('cancel');
            
            $scope.limiteAprovacao.limitesAprovacoesValores[$scope.limiteAprovacaoValorChecked.$index].dataInativo = $filter('date')(new Date(), 'dd/MM/yyyy');
            $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
          }
        });
      }
    };
    
	$scope.grupoAlcadaSource = [];	
	$scope.grupoNivelAutoridadeSource = [];
    
	$scope.filterCriteriaNivelAutoridade = {
		start : 1,
		dir : 'asc',
		sort : 'id',
		limit : 10,
		fields: ['id', 'nome']
	};
	
	$scope.filterCriteriaAlcada = {
		start : 1,
		dir : 'asc',
		sort : 'id',
		limit : 10,
		fields: ['id', 'nome']			
	};
    
	//Método responsável por listar os niveis de autoridade do sistema 
	$scope.findNivelAutoridadeSource = function(){		   
		$scope.grupoNivelAutoridadeSource = [];
		NivelAutoridadeRepository.getListPage($scope.filterCriteriaNivelAutoridade).then(function(result) {
			//Percorre a lista de nivelAutoridade para montar a lista de grupoNiveiAutoridade
			result.originalElement.objects.forEach(function (item, index) {					 
				var grupoNivelAutoridade = {};					 
				grupoNivelAutoridade.nivelAutoridade = item;			
				$scope.grupoNivelAutoridadeSource.push(grupoNivelAutoridade);
				$timeout(function(){
					$scope.grupoNivelAutoridadeSource =  $filter('idNotEqualNivelAutoridadeSourcePickList')($scope.grupoNivelAutoridadeSource, $scope.limiteAprovacao.niveisAutoridades);
				});
			});
	   });   
	};
	   
	//Método responsável por listar as alcadas
	$scope.findAlcadaSource = function(){		  
		$scope.grupoAlcadaSource = [];
		AlcadaRepository.getListPage($scope.filterCriteriaAlcada).then(function(result) {
			//Percorre a lista de alcada para montar a lista de grupoAlcada
			result.originalElement.objects.forEach(function (item, index) {					 
				var grupoAlcada = {};					 
				grupoAlcada.alcada = item;					 
				$scope.grupoAlcadaSource.push(grupoAlcada);
				$timeout(function(){
					$scope.grupoAlcadaSource =  $filter('idNotEqualAlcadaSourcePickList')($scope.grupoAlcadaSource, $scope.limiteAprovacao.alcadas);
				});
			});	 
		});			   
	}; 
    
}]);